import fedmsg.meta.base
from fedmsg_meta_fedora_infrastructure.fasshim import avatar_url


class AbstractCoprConglomerator(fedmsg.meta.base.BaseConglomerator):
    def can_handle(self, msg, **config):
        return '.copr.' in msg['topic']

    def merge(self, constituents, subject, **config):
        ms = constituents  # shorthand

        agents = set([m['msg']['user'] for m in ms])
        coprs = set([m['msg']['copr'] for m in ms])
        count = len([1 for m in ms
                      if m['topic'].endswith('copr.build.start')])

        if count > 0:
            subtitle = '{agents} kicked off {count} {rebuild_predicate} ' + \
                'of the {coprs} {copr_predicate}'
        else:
            # If count is zero, then there are zero start messages in the
            # constituents list, so change tack and render "finishes" instead
            # of "starts".
            count = len(ms)
            subtitle = 'The {coprs} {copr_predicate} finished ' + \
                '{count} {rebuild_predicate} by {agents}'

        rebuild_predicate = 'rebuilds' if count > 1 else 'rebuild'
        copr_predicate = 'coprs' if len(coprs) > 1 else 'copr'

        agents = self.list_to_series(agents)
        coprs = self.list_to_series(coprs)

        tmpl = self.produce_template(constituents, subject, **config)
        tmpl['subtitle'] = subtitle.format(
            agents=agents, count=count, coprs=coprs,
            copr_predicate=copr_predicate, rebuild_predicate=rebuild_predicate)
        tmpl['subjective'] = tmpl['subtitle']

        default = tmpl['icon']

        tmpl['secondary_icon'] = self.get_secondary_icon(constituents, default)
        tmpl['link'] = self.get_link(constituents)

        return tmpl


class ByCopr(AbstractCoprConglomerator):
    def matches(self, a, b, **config):
        a, b = a['msg'], b['msg']
        if a['copr'] != b['copr']:
            return False
        return True

    def get_secondary_icon(self, constituents, default):
        agents = set([m['msg']['user'] for m in constituents])
        if len(agents) == 1:
            user = constituents[0]['msg']['user']
            return avatar_url(user)
        else:
            return 'https://apps.fedoraproject.org/img/icons/copr.png'

    def get_link(self, constituents):
        owner = constituents[0]['msg']['owner']
        copr = constituents[0]['msg']['copr']
        return 'https://copr.fedoraproject.org/coprs/%s/%s/' % (owner, copr)


class ByUser(AbstractCoprConglomerator):
    def matches(self, a, b, **config):
        a, b = a['msg'], b['msg']
        if a['user'] != b['user']:
            return False
        return True

    def get_secondary_icon(self, constituents, default):
        user = constituents[0]['msg']['user']
        return avatar_url(user)

    def get_link(self, constituents):
        user = constituents[0]['msg']['user']
        return 'https://copr.fedoraproject.org/coprs/' + user



import fedmsg.meta.base
from fedmsg_meta_fedora_infrastructure.fasshim import avatar_url, email2fas


class AbstractPagureTicketConglomerator(fedmsg.meta.base.BaseConglomerator):

    def can_handle(self, msg, **config):
        target = 'pagure.{entity_name}.'.format(entity_name=self.entity_name)
        return target in msg['topic']

    def merge(self, constituents, subject, **config):
        ms = constituents  # shorthand

        count = len(ms)
        agents = set([m['msg']['agent'] for m in ms])
        idx = ms[0]['msg'][self.entity_key]['id']
        repo = self.get_repo(ms[0])
        subtitle = '{agents} interacted with {entity_name} #{idx} of project "{repo}" {count} times'

        agents = self.list_to_series(agents)

        tmpl = self.produce_template(constituents, subject, **config)
        tmpl['subtitle'] = subtitle.format(
            agents=agents,
            entity_name=self.entity_name,
            repo=repo,
            idx=idx,
            count=count,
        )
        tmpl['subjective'] = tmpl['subtitle']

        tmpl['secondary_icon'] = avatar_url(ms[0]['msg']['agent'])
        link_template = 'https://pagure.io/{repo}/{entity_name}/{idx}'
        tmpl['link'] = link_template.format(
            repo=repo,
            entity_name=self.entity_name,
            idx=idx,
        )

        return tmpl


class ByPR(AbstractPagureTicketConglomerator):
    entity_key = 'pullrequest'
    entity_name = 'pull-request'

    def matches(self, a, b, **config):
        a, b = a['msg'][self.entity_key], b['msg'][self.entity_key]
        return (
            a['id'] == b['id']
            and a['project']['name'] == b['project']['name']
        )

    def get_repo(self, msg):
        return msg['msg']['pullrequest']['project']['name']


class ByIssue(AbstractPagureTicketConglomerator):
    entity_key = 'issue'
    entity_name = 'issue'

    def matches(self, a, b, **config):
        a, b = a['msg'], b['msg']
        return (
            a[self.entity_key]['id'] == b[self.entity_key]['id']
            and a['project']['name'] == b['project']['name']
        )

    def get_repo(self, msg):
        return msg['msg']['project']['name']


class ByNewStyleCommit(fedmsg.meta.base.BaseConglomerator):
    def can_handle(self, msg, **config):
        return '.pagure.git.receive' in msg['topic'] and 'commit' not  in msg['msg']

    def matches(self, a, b, **config):
        a, b = a['msg'], b['msg']
        return a['repo']['name'] == b['repo']['name']

    def merge(self, constituents, subject, **config):
        ms = constituents  # shorthand

        count = sum([m['msg']['total_commits'] for m in ms])
        agents = set([m['msg']['agent'] for m in ms])
        branches = [m['msg']['branch'].replace('refs/heads/', '') for m in ms]
        repo = ms[0]['msg']['repo']['name']
        subtitle = '{agents} pushed {count} commits to {repo} ({branches})'
        agents = self.list_to_series(agents)
        branches = self.list_to_series(branches)

        tmpl = self.produce_template(constituents, subject, **config)
        tmpl['subtitle'] = subtitle.format(
            agents=agents,
            repo=repo,
            count=count,
            branches=branches,
        )
        tmpl['subjective'] = tmpl['subtitle']

        tmpl['secondary_icon'] = avatar_url(ms[0]['msg']['agent'])
        link_template = 'https://pagure.io/{repo}/commits'
        tmpl['link'] = link_template.format(repo=repo)

        return tmpl


class ByOldStyleCommit(fedmsg.meta.base.BaseConglomerator):
    def can_handle(self, msg, **config):
        return '.pagure.git.receive' in msg['topic'] and 'commit' in msg['msg']

    def matches(self, a, b, **config):
        a, b = a['msg'], b['msg']
        return a['commit']['path'] == b['commit']['path']

    def merge(self, constituents, subject, **config):
        ms = constituents  # shorthand

        count = len(ms)
        agents = set([
            email2fas(m['msg']['commit']['email'], **config) for m in ms
        ])
        repo = ms[0]['msg']['commit']['repo']['name']
        subtitle = '{agents} pushed {count} commits to the {repo} project'
        agents = self.list_to_series(agents)

        tmpl = self.produce_template(constituents, subject, **config)
        tmpl['subtitle'] = subtitle.format(
            agents=agents,
            repo=repo,
            count=count,
        )
        tmpl['subjective'] = tmpl['subtitle']

        tmpl['secondary_icon'] = avatar_url(list(agents)[0])
        link_template = 'https://pagure.io/{repo}/commits'
        tmpl['link'] = link_template.format(repo=repo)

        return tmpl



# This file is part of fedmsg.
# Copyright (C) 2012-2014 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
""" Tests for fedmsg.meta """

import unittest

from fedmsg_meta_fedora_infrastructure.tests.bodhi import *
from fedmsg_meta_fedora_infrastructure.tests.compose import *
from fedmsg_meta_fedora_infrastructure.tests.compose2 import *
from fedmsg_meta_fedora_infrastructure.tests.pkgdb import *
from fedmsg_meta_fedora_infrastructure.tests.planet import *
from fedmsg_meta_fedora_infrastructure.tests.buildsys import *
from fedmsg_meta_fedora_infrastructure.tests.askbot import *
from fedmsg_meta_fedora_infrastructure.tests.tagger import *
from fedmsg_meta_fedora_infrastructure.tests.trac import *
from fedmsg_meta_fedora_infrastructure.tests.mailman3 import *
from fedmsg_meta_fedora_infrastructure.tests.badges import *
from fedmsg_meta_fedora_infrastructure.tests.ansible import *
from fedmsg_meta_fedora_infrastructure.tests.scm import *
from fedmsg_meta_fedora_infrastructure.tests.datanommer import *
from fedmsg_meta_fedora_infrastructure.tests.nuancier import *
from fedmsg_meta_fedora_infrastructure.tests.fedocal import *
from fedmsg_meta_fedora_infrastructure.tests.coprs import *
from fedmsg_meta_fedora_infrastructure.tests.anitya import *
from fedmsg_meta_fedora_infrastructure.tests.summershum import *
from fedmsg_meta_fedora_infrastructure.tests.jenkins import *
from fedmsg_meta_fedora_infrastructure.tests.github import *
from fedmsg_meta_fedora_infrastructure.tests.ftpsync import *
from fedmsg_meta_fedora_infrastructure.tests.bz import *
from fedmsg_meta_fedora_infrastructure.tests.elections import *
from fedmsg_meta_fedora_infrastructure.tests.fmn import *
from fedmsg_meta_fedora_infrastructure.tests.fedimg import *
from fedmsg_meta_fedora_infrastructure.tests.kerneltest import *
from fedmsg_meta_fedora_infrastructure.tests.koschei import *
from fedmsg_meta_fedora_infrastructure.tests.fas import *
from fedmsg_meta_fedora_infrastructure.tests.hotness import *
from fedmsg_meta_fedora_infrastructure.tests.mm2 import *
from fedmsg_meta_fedora_infrastructure.tests.zanata import *
from fedmsg_meta_fedora_infrastructure.tests.zodbot import *
from fedmsg_meta_fedora_infrastructure.tests.faf import *
from fedmsg_meta_fedora_infrastructure.tests.pagure import *
from fedmsg_meta_fedora_infrastructure.tests.autocloud import *
from fedmsg_meta_fedora_infrastructure.tests.infragit import *
from fedmsg_meta_fedora_infrastructure.tests.taskotron import *
from fedmsg_meta_fedora_infrastructure.tests.releng import *
from fedmsg_meta_fedora_infrastructure.tests.mdapi import *
from fedmsg_meta_fedora_infrastructure.tests.nagios import *
from fedmsg_meta_fedora_infrastructure.tests.openqa import *
from fedmsg_meta_fedora_infrastructure.tests.pdc import *

from fedmsg_meta_fedora_infrastructure.tests.base import Base

from fedmsg_meta_fedora_infrastructure.tests.common import add_doc


class TestMediaWikiEdit(Base):
    """ Fedora's `Wiki <https://fedoraproject.org/wiki>`_ has a fedmsg hook
    that publishes messages like this one when a user edits a page.
    """
    expected_title = "wiki.article.edit"
    expected_subti = 'Ralph made a wiki edit to "Messaging SIG"'
    expected_link = "https://this-is-a-link.org"
    expected_icon = "https://fedoraproject.org/w/skins/common/" + \
        "images/mediawiki.png"
    expected_usernames = set(['ralph'])
    expected_objects = set(['Messaging SIG-page'])

    msg = {
        "topic": "org.fedoraproject.stg.wiki.article.edit",
        "msg": {
            "watch_this": None,
            "base_rev_id": False,
            "title": "Messaging SIG",
            "minor_edit": 0,
            "text": "The diff goes here...",
            "section_anchor": None,
            "summary": "/* Mission */ ",
            "user": "Ralph",
            "revision": None,
            "url": "http://this-is-a-link.org"
        },
        "timestamp": 1344350200
    }


class TestMediaWikiUpload(Base):
    """ Fedora's `Wiki <https://fedoraproject.org/wiki>`_ hook also publishes
    messages when a user upload some media (like a video or a picture).
    """
    expected_title = "wiki.upload.complete"
    expected_subti = 'Ralph uploaded File:Cat.jpg to the wiki: ' + \
        '"This is a beautiful cat..."'
    expected_icon = "https://fedoraproject.org/w/skins/common/" + \
        "images/mediawiki.png"
    expected_usernames = set(['ralph'])
    expected_objects = set(['w/uploads/d/d1/Cat.jpg'])

    msg = {
        "topic": "org.fedoraproject.stg.wiki.upload.complete",
        "msg": {
            "user_id": 8306,
            "description": "This is a beautiful cat",
            "title": {
                "mPrefixedText": "File:Cat.jpg",
            },
            "user_text": "Ralph",
            "minor_mime": "jpeg",
            "url": "/w/uploads/d/d1/Cat.jpg",
            "file_exists": True,
            "mime": "image/jpeg",
            "major_mime": "image",
            "media_type": "BITMAP",
            "size": 295667
        },
        "timestamp": 1344361406
    }


class TestPkgdb2BrMassStart(Base):
    """ There is a script called ``pkgdb2branch`` that gets run by an SCM
    admin as part of the new package process.  Messages on this topic are
    emitted from that script when it is instructed to carry out a "mass
    branch" of all packages.
    """
    expected_title = "git.mass_branch.start"
    expected_subti = "dgilmore started a mass branch"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "168a37a06ac9b4dbcf4f2e0e11de6a1332c7aead2313b04016cea0c5a848263e" + \
        "?s=64&d=retro"
    expected_usernames = set(['dgilmore'])

    msg = {
        "i": 1,
        "timestamp": 1344350850.8867381,
        "topic": "org.fedoraproject.prod.git.mass_branch.start",
        "msg": {
            "agent": "dgilmore",
        },
    }


class TestPkgdb2BrMassComplete(Base):
    """ There is a script called ``pkgdb2branch`` that gets run by an SCM
    admin as part of the new package process.  Messages on this topic are
    emitted from that script when it **finishes** a "mass branch".
    """
    expected_title = "git.mass_branch.complete"
    expected_subti = "mass branch started by dgilmore completed"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "168a37a06ac9b4dbcf4f2e0e11de6a1332c7aead2313b04016cea0c5a848263e" + \
        "?s=64&d=retro"
    expected_usernames = set(['dgilmore'])

    msg = {
        "i": 1,
        "timestamp": 1344350850.8867381,
        "topic": "org.fedoraproject.prod.git.mass_branch.complete",
        "msg": {
            "agent": "dgilmore",
        },
    }


class TestPkgdb2BrRunStart(Base):
    """ There is a script called ``pkgdb2branch`` that gets run by an SCM
    admin as part of the new package process.  Typically, when an `SCM Admin
    Request <https://fedoraproject.org/wiki/Package_SCM_admin_requests>`_ is
    approved, the scm admin will add the new package or branch to the package
    database.  *After that*, the scm admin will run ``pkgdb2branch`` to create
    the branch in git on the file system.  Messages of **this** topic are
    published when that process **begins**.
    """
    expected_title = "git.pkgdb2branch.start"
    expected_subti = "limburgher started a run of pkgdb2branch"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "67fc36d5974ea45ec922a650dc117d83acf0a1f646cd17ba80e9e0fe6e2ed164" + \
        "?s=64&d=retro"
    expected_usernames = set(['limburgher'])

    msg = {
        "i": 1,
        "timestamp": 1344350850.8867381,
        "topic": "org.fedoraproject.prod.git.pkgdb2branch.start",
        "msg": {
            "agent": "limburgher",
        },
    }


class TestPkgdb2BrRunComplete(Base):
    """ There is a script called ``pkgdb2branch`` that gets run by an SCM
    admin as part of the new package process.  Typically, when an `SCM Admin
    Request <https://fedoraproject.org/wiki/Package_SCM_admin_requests>`_ is
    approved, the scm admin will add the new package or branch to the package
    database.  *After that*, the scm admin will run ``pkgdb2branch`` to create
    the branch in git on the file system.  Messages of **this** topic are
    published when that process **completes**.
    """
    expected_title = "git.pkgdb2branch.complete"
    expected_subti = "run of pkgdb2branch started by limburgher completed"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "67fc36d5974ea45ec922a650dc117d83acf0a1f646cd17ba80e9e0fe6e2ed164" + \
        "?s=64&d=retro"
    expected_usernames = set(['limburgher'])
    expected_packages = set(['nethack'])
    expected_objects = set(['nethack/__git__'])

    msg = {
        "i": 1,
        "timestamp": 1344350850.8867381,
        "topic": "org.fedoraproject.prod.git.pkgdb2branch.complete",
        "msg": {
            "agent": "limburgher",
            "unbranchedPackages": [],
            "branchedPackages": ["nethack"],
        },
    }


class TestPkgdb2BrRunCompleteWithError(Base):
    """ There is a script called ``pkgdb2branch`` that gets run by an SCM
    admin as part of the new package process.  Typically, when an `SCM Admin
    Request <https://fedoraproject.org/wiki/Package_SCM_admin_requests>`_ is
    approved, the scm admin will add the new package or branch to the package
    database.  *After that*, the scm admin will run ``pkgdb2branch`` to create
    the branch in git on the file system.  Messages of **this** topic are
    published when that process **completes**.

    *Sometimes* that process can produce errors.  Here's an example of a
    message from a failed ``pkgdb2branch`` run.
    """
    expected_title = "git.pkgdb2branch.complete"
    expected_subti = "run of pkgdb2branch started by limburgher completed" + \
        " with 1 error"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "67fc36d5974ea45ec922a650dc117d83acf0a1f646cd17ba80e9e0fe6e2ed164" + \
        "?s=64&d=retro"
    expected_usernames = set(['limburgher'])
    expected_packages = set(['foo'])
    expected_objects = set(['foo/__git__'])

    msg = {
        "i": 1,
        "timestamp": 1344350850.8867381,
        "topic": "org.fedoraproject.prod.git.pkgdb2branch.complete",
        "msg": {
            "agent": "limburgher",
            "unbranchedPackages": ['foo'],
            "branchedPackages": None,
        },
    }


class TestPkgdb2BrRunCompleteWithErrors(Base):
    """ There is a script called ``pkgdb2branch`` that gets run by an SCM
    admin as part of the new package process.  Typically, when an `SCM Admin
    Request <https://fedoraproject.org/wiki/Package_SCM_admin_requests>`_ is
    approved, the scm admin will add the new package or branch to the package
    database.  *After that*, the scm admin will run ``pkgdb2branch`` to create
    the branch in git on the file system.  Messages of **this** topic are
    published when that process **completes**.

    *Sometimes* that process can produce errors.  Here's an example of a
    message from a failed ``pkgdb2branch`` run (on multiple packages)
    """
    expected_title = "git.pkgdb2branch.complete"
    expected_subti = "run of pkgdb2branch started by limburgher completed" + \
        " with 2 errors"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "67fc36d5974ea45ec922a650dc117d83acf0a1f646cd17ba80e9e0fe6e2ed164" + \
        "?s=64&d=retro"
    expected_usernames = set(['limburgher'])
    expected_packages = set(['foo', 'bar'])
    expected_objects = set(['foo/__git__', 'bar/__git__'])

    msg = {
        "i": 1,
        "timestamp": 1344350850.8867381,
        "topic": "org.fedoraproject.prod.git.pkgdb2branch.complete",
        "msg": {
            "agent": "limburgher",
            "unbranchedPackages": ['foo', 'bar'],
            "branchedPackages": [],
        },
    }


class TestPkgdb2BrCreateLegacy(Base):
    """ Support old school pkgdb2branch messages.  This don't get published
    anymore, but :mod:`fedmsg.meta` needs to support them since they are
    stored *forever* in datanommer.
    """

    expected_title = "git.branch.valgrind.master"
    expected_subti = \
        "limburgher created branch 'master' for the 'valgrind' package"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "67fc36d5974ea45ec922a650dc117d83acf0a1f646cd17ba80e9e0fe6e2ed164" + \
        "?s=64&d=retro"
    expected_link = \
        "http://pkgs.fedoraproject.org/cgit/valgrind.git/log/?h=master"
    expected_usernames = set(['limburgher'])
    expected_packages = set(['valgrind'])
    expected_objects = set(['valgrind/__git__'])

    msg = {
        "i": 1,
        "timestamp": 1344350850.8867381,
        "topic": "org.fedoraproject.prod.git.branch.valgrind.master",
        "msg": {
            "agent": "limburgher",
        },
    }


class TestPkgdb2BrCreate(Base):
    """ There is a script called ``pkgdb2branch`` that gets run by an SCM
    admin as part of the new package process.  Typically, when an `SCM Admin
    Request <https://fedoraproject.org/wiki/Package_SCM_admin_requests>`_ is
    approved, the scm admin will add the new package or branch to the package
    database.  *After that*, the scm admin will run ``pkgdb2branch`` to create
    the branch in git on the file system.  Messages of **this** topic are
    published `for each new branch` that that process **creates**.
    """
    expected_title = "git.branch"
    expected_subti = \
        "limburgher created branch 'master' for the 'valgrind' package"
    expected_link = \
        "http://pkgs.fedoraproject.org/cgit/valgrind.git/log/?h=master"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "67fc36d5974ea45ec922a650dc117d83acf0a1f646cd17ba80e9e0fe6e2ed164" + \
        "?s=64&d=retro"
    expected_usernames = set(['limburgher'])
    expected_packages = set(['valgrind'])
    expected_objects = set(['valgrind/__git__'])

    msg = {
        "i": 1,
        "timestamp": 1344350850.8867381,
        "topic": "org.fedoraproject.prod.git.branch",
        "msg": {
            "agent": "limburgher",
            "name": "valgrind",
            "branch": "master",
        },
    }


class TestLookaside(Base):
    """ Messages like this one are published when **new sources** are
    uploaded to the "lookaside cache".
    """

    expected_title = "git.lookaside.new"
    expected_subti = 'jnovy uploaded pst-diffraction.doc.tar.xz for texlive'
    expected_icon = "https://apps.fedoraproject.org/img/icons/git-logo.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "e0e8e0c4d995109cdac8ae4eb5766a73cf09c7a8d2d8bac57f761e6223ca094b?s=64&" + \
        "d=retro"
    expected_link = 'http://pkgs.fedoraproject.org/lookaside/pkgs/' + \
        'texlive/pst-diffraction.doc.tar.xz/' + \
        'dacad985394b3977f9dcf0c75f51a357/' + \
        'pst-diffraction.doc.tar.xz'
    expected_long_form = \
        'dacad985394b3977f9dcf0c75f51a357  pst-diffraction.doc.tar.xz'
    expected_usernames = set(['jnovy'])
    expected_packages = set(['texlive'])
    expected_objects = set(['texlive/pst-diffraction.doc.tar.xz'])

    msg = {
        "i": 1,
        "timestamp": 1349197866.215465,
        "topic": "org.fedoraproject.prod.git.lookaside.new",
        "msg": {
            "agent": "jnovy",
            "md5sum": "dacad985394b3977f9dcf0c75f51a357",
            "name": "texlive",
            "filename": "pst-diffraction.doc.tar.xz"
        }
    }


class TestLookasideLegacy(Base):
    """ Support oldschool lookaside messages.  :( """

    expected_title = "git.lookaside.texlive.new"
    expected_subti = 'jnovy uploaded pst-diffraction.doc.tar.xz for texlive'
    expected_icon = "https://apps.fedoraproject.org/img/icons/git-logo.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "e0e8e0c4d995109cdac8ae4eb5766a73cf09c7a8d2d8bac57f761e6223ca094b?s=64&" + \
        "d=retro"
    expected_link = 'http://pkgs.fedoraproject.org/lookaside/pkgs/' + \
        'texlive/pst-diffraction.doc.tar.xz/' + \
        'dacad985394b3977f9dcf0c75f51a357/' + \
        'pst-diffraction.doc.tar.xz'
    expected_usernames = set(['jnovy'])
    expected_packages = set(['texlive'])
    expected_objects = set(['texlive/pst-diffraction.doc.tar.xz'])

    msg = {
        "i": 1,
        "timestamp": 1349197866.215465,
        "topic": "org.fedoraproject.prod.git.lookaside.texlive.new",
        "msg": {
            "agent": "jnovy",
            "md5sum": "dacad985394b3977f9dcf0c75f51a357",
            "name": "texlive",
            "filename": "pst-diffraction.doc.tar.xz"
        }
    }


add_doc(locals())

if __name__ == '__main__':
    unittest.main()



# This file is part of fedmsg.
# Copyright (C) 2013, 2014 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#           Luke Macken <lmacken@redhat.com>
#           Pierre-Yves Chibon <pingou@pingoured.fr>
#

from fedmsg_meta_fedora_infrastructure.fasshim import \
        avatar_url, avatar_url_from_email, email2fas
from fedmsg_meta_fedora_infrastructure import BaseProcessor
import fedmsg.meta.base

class AnityaProcessor(BaseProcessor):
    topic_prefix_re = 'org\\.release-monitoring\\.(dev|stg|prod)'

    __name__ = "anitya"
    __description__ = "Upstream Release Monitoring"
    __link__ = "https://release-monitoring.org"
    __docs__ = "https://github.com/fedora-infra/anitya"
    __obj__ = "Upstream Releases"
    __icon__ = ("https://apps.fedoraproject.org/packages/"
                "images/icons/package_128x128.png")

    def _get_user(self, msg, **config):
        try:
            email = msg['msg']['message']['agent']
        except KeyError:
            return msg.get('username', 'anitya')
        else:
            if email.endswith('@fedoraproject.org'):
                return email.split('@fedoraproject.org')[0]
            else:
                return email2fas(email, **config)

    def link(self, msg, **config):
        if msg['msg']['project']:
            proj = msg['msg']['project']['id']
            return "https://release-monitoring.org/project/%s/" % proj
        else:
            return "https://release-monitoring.org/distros"

        return None

    def subtitle(self, msg, **config):
        user = self._get_user(msg, **config)
        if 'project.map.new' in msg['topic']:
            project = msg['msg']['project']['name']
            distro = msg['msg']['distro']['name']
            new = msg['msg']['message']['new']
            tmpl = self._(
                '{user} mapped the name of "{project}" in {distro} '
                'to "{new}"'
            )
            return tmpl.format(
                user=user, project=project, distro=distro, new=new)
        elif 'project.map.update' in msg['topic']:
            project = msg['msg']['project']['name']
            distro = msg['msg']['distro']['name']
            old = msg['msg']['message']['prev']
            new = msg['msg']['message']['new']
            tmpl = self._(
                '{user} updated the name of "{project}" in '
                '"{distro}" from "{old}" to "{new}"')
            return tmpl.format(
                user=user, project=project, distro=distro, new=new, old=old)
        elif 'distro.add' in msg['topic']:
            distro = msg['msg']['distro']['name']
            tmpl = self._(
                '{user} added the distro named "{distro}" to anitya')
            return tmpl.format(user=user, distro=distro)
        elif 'distro.edit' in msg['topic']:
            old = msg['msg']['message']['old']
            new = msg['msg']['message']['new']
            tmpl = self._(
                '{user} changed a distro name from "{old}" to "{new}"')
            return tmpl.format(user=user, old=old, new=new)
        elif 'project.add.tried' in msg['topic']:
            project = msg['msg']['project']['name']
            tmpl = self._('{user} tried to add the project '
                          '"{project}" to anitya '
                          '(but it already exists there)')
            return tmpl.format(user=user, project=project)
        elif 'project.add' in msg['topic']:
            project = msg['msg']['project']['name']
            tmpl = self._('{user} added the project "{project}" to anitya')
            return tmpl.format(user=user, project=project)
        elif 'project.edit' in msg['topic']:
            project = msg['msg']['project']['name']
            fields = ', '.join(msg['msg']['message']['fields'])
            tmpl = self._(
                '{user} edited the following fields of the '
                '"{project}" project: {fields}')
            return tmpl.format(user=user, project=project, fields=fields)
        elif 'project.remove' in msg['topic']:
            project = msg['msg']['project']['name']
            tmpl = self._('{user} deleted the "{project}" project')
            return tmpl.format(user=user, project=project)
        elif 'project.map.remove' in msg['topic']:
            project = msg['msg']['project']['name']
            distro = msg['msg']['message']['distro']
            tmpl = self._(
                '{user} deleted the mapping of "{project}" project '
                'on "{distro}"')
            return tmpl.format(user=user, project=project, distro=distro)
        elif 'project.version.update' in msg['topic']:
            project = msg['msg']['project']['name']

            if 'message' in msg['msg']:
                message = msg['msg']['message']
            else:
                message = msg['msg']

            # If this project is mapped to one or more Fedora package,
            # then just add the mapped packages to make
            # everyone's emails more readable.
            # https://github.com/fedora-infra/the-new-hotness/issues/21
            packages = []
            for package in message.get('packages', []):
                if package['distro'] == 'Fedora':
                    packages.append(package['package_name'])
            packages = fedmsg.meta.base.BaseConglomerator.list_to_series(packages)
            odd = message.get('odd_change', False)
            old = message['old_version']
            new = message['upstream_version']
            tmpl = self._(
                'A new version of "{project}" has been detected:  '
                '"{new}", packaged as "{packages}"')
            if old and not odd:
                tmpl = self._(
                    'A new version of "{project}" has been detected:  '
                    '"{new}" newer than "{old}", packaged as "{packages}"')

            return tmpl.format(project=project, new=new, old=old, packages=packages)
        elif 'project.version.remove' in msg['topic']:
            project = msg['msg']['project']['name']
            version = msg['msg']['message']['version']
            tmpl = self._(
                '{user} deleted the version {version} of "{project}"')
            return tmpl.format(user=user, project=project, version=version)
        elif 'distro.remove' in msg['topic']:
            project = msg['msg']['distro']['name']
            tmpl = self._(
                '{user} deleted the distro "{project}"')
            return tmpl.format(user=user, project=project)
        elif msg['topic'].endswith('project.flag'):
            project = msg['msg']['project']['name']
            tmpl = self._(
                '{user} flagged the project "{project}"')
            return tmpl.format(user=user, project=project)
        else:
            pass

    def secondary_icon(self, msg, **config):
        username = self._get_user(msg, **config)
        packages = self.packages(msg, **config)
        if username:
            if '@' in username:
                return avatar_url_from_email(username)
            else:
                return avatar_url(username)
        elif packages:
            tmpl = 'https://apps.fedoraproject.org/packages/' + \
                'images/icons/%s.png'
            return tmpl % list(packages)[0]
        else:
            return None

    def usernames(self, msg, **config):
        username = self._get_user(msg, **config)
        if username and '@' not in username:
            return set([username])
        else:
            return set([])

    def objects(self, msg, **config):
        if 'project.map.update' in msg['topic']:
            distro = msg['msg']['distro']['name']
            return set([
                'mappings/%s/%s' % (distro, msg['msg']['message']['new']),
                'mappings/%s/%s' % (distro, msg['msg']['message']['prev']),
                'distros/%s' % distro,
                'projects/%s' % msg['msg']['project']['name'],
            ])
        elif 'project.map.new' in msg['topic']:
            distro = msg['msg']['distro']['name']
            return set([
                'mappings/%s/%s' % (distro, msg['msg']['message']['new']),
                'distros/%s' % distro,
                'projects/%s' % msg['msg']['project']['name'],
            ])
        elif 'project.version.remove' in msg['topic']:
            return set(['projects/%s' % msg['msg']['project']['name']])
        elif 'project.remove' in msg['topic']:
            return set(['projects/%s' % msg['msg']['project']['name']])
        elif 'project.map.remove' in msg['topic']:
            return set(['projects/%s' % msg['msg']['project']['name']])
        elif 'project.version.update' in msg['topic']:
            return set(['projects/%s' % msg['msg']['project']['name']])
        elif 'project.edit' in msg['topic']:
            return set(['projects/%s' % msg['msg']['project']['name']])
        elif 'project.add.tried' in msg['topic']:
            return set(['projects/%s' % msg['msg']['project']['name']])
        elif 'project.add' in msg['topic']:
            return set(['projects/%s' % msg['msg']['project']['name']])
        elif msg['topic'].endswith('project.flag'):
            return set(['projects/%s' % msg['msg']['project']['name']])
        elif 'distro.add' in msg['topic']:
            distro = msg['msg']['distro']['name']
            return set(['distros/%s' % distro])
        elif 'distro.edit' in msg['topic']:
            distro_old = msg['msg']['message']['old']
            distro_new = msg['msg']['message']['new']
            return set([
                'distros/%s' % distro_old,
                'distros/%s' % distro_new,
            ])
        elif 'distro.remove' in msg['topic']:
            distro = msg['msg']['distro']['name']
            return set(['distros/%s' % distro])

        return set([])

    def packages(self, msg, **config):
        if 'project.map.update' in msg['topic']:
            if msg['msg']['distro']['name'].lower() == 'fedora':
                return set([
                    msg['msg']['message']['new'],
                    msg['msg']['message']['prev'],
                ])
        elif 'project.map.new' in msg['topic']:
            if msg['msg']['distro']['name'].lower() == 'fedora':
                return set([msg['msg']['message']['new']])
        elif 'project.version.update' in msg['topic']:
            if 'message' in msg['msg']:
                packages = msg['msg']['message']['packages']
            else:
                packages = msg['msg']['packages']
            return set([
                pkg['package_name']
                for pkg in packages
                if pkg['distro'].lower() == 'fedora'
            ])
        elif msg['topic'].endswith('project.flag'):
            packages = msg['msg'].get('packages', [])
            return set([
                pkg['package_name']
                for pkg in packages
                if pkg['distro'].lower() == 'fedora'
            ])

        return set([])



# This file is part of fedmsg.
# Copyright (C) 2012, 2013 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#           Luke Macken <lmacken@redhat.com>

from fedmsg_meta_fedora_infrastructure.fasshim import avatar_url
from fedmsg_meta_fedora_infrastructure import BaseProcessor


class BadgesProcessor(BaseProcessor):
    __name__ = "fedbadges"
    __description__ = "the Fedora Open Badges System"
    __link__ = "https://badges.fedoraproject.org/"
    __icon__ = "https://apps.fedoraproject.org/img/icons/badges.png"
    __docs__ = "https://fedoraproject.org/wiki/Open_Badges"
    __obj__ = "New Badges"

    def _get_user(self, msg):
        # Handle the difference between badge.award and person.rank.advance.
        if 'user' in msg['msg']:
            return msg['msg']['user']['username']
        else:
            return msg['msg']['person']['nickname']

    def link(self, msg, **config):
        username = self._get_user(msg)
        return "https://badges.fedoraproject.org/user/%s" % username

    def long_form(self, msg, **config):
        if 'badge.award' in msg['topic']:
            return msg['msg']['badge']['description']

    def subtitle(self, msg, **config):
        user = self._get_user(msg)
        if 'badge.award' in msg['topic']:
            name = msg['msg']['badge']['name']
            tmpl = self._('{user} has been awarded the "{name}" badge')
            return tmpl.format(user=user, name=name)
        elif 'person.rank.advance' in msg['topic']:
            rank = msg['msg']['person']['rank']
            tmpl = self._('{user} moved to position {rank} '
                          'on the badges leaderboard')
            return tmpl.format(user=user, rank=rank)
        elif 'person.login.first' in msg['topic']:
            tmpl = self._('{user} logged in to badges.fedoraproject.org '
                          'for the first time')
            return tmpl.format(user=user)
        else:
            pass

    def icon(self, msg, **config):
        if 'badge.award' in msg['topic']:
            return msg['msg']['badge']['image_url']
        else:
            return super(BadgesProcessor, self).icon(msg, **config)

    def secondary_icon(self, msg, **config):
        return avatar_url(self._get_user(msg))

    def usernames(self, msg, **config):
        return set([self._get_user(msg)])

    def objects(self, msg, **config):
        if 'badge.award' in msg['topic']:
            return set([msg['msg']['badge']['name'].lower().replace(' ', '-')])
        else:
            return set([])



# This file is part of fedmsg.
# Copyright (C) 2014 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
""" Tests for kerneltest-harness messages """

import unittest

from fedmsg_meta_fedora_infrastructure.tests.base import Base

from .common import add_doc


class TestKerneltestNew(Base):
    """ As part of the `kernel testing initiative
    <https://fedoraproject.org/wiki/KernelTestingInitiative>`_, we built an
    `app <https://apps.fedoraproject.org/kerneltest>`_ that lets both real
    users and automated systems provide test feedback on new kernel builds and
    releases.

    *These* kinds of messages are published whenever either an automated system
    or a real user **uploads a new test result**.
    """
    expected_title = "kerneltest.upload.new"
    expected_subti = "jforbes ran a test of " + \
        "3.16.0-0.rc3.git3.1.fc21.x86_64 (PASS)"
    expected_link = "https://apps.fedoraproject.org/kerneltest/kernel/" + \
        "3.16.0-0.rc3.git3.1.fc21.x86_64"
    expected_icon = "https://apps.fedoraproject.org/img/icons/tux.png"
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "03ead14065750aad5e4eb5ee42f7c60fb1e50ea2f5ba2dc9a89639bc381796da"
        "?s=64&d=retro")
    expected_packages = set(['kernel'])
    expected_usernames = set(['jforbes'])
    expected_objects = set([
        'kernel/3.16.0-0.rc3.git3.1.fc21.x86_64',
    ])
    msg = {
        u'username': u'apache',
        u'i': 1,
        u'timestamp': 1404746131,
        u'msg_id': u'2014-4c25f684-c965-4003-b22d-57cda0e2e12c',
        u'crypto': u'x509',
        u'topic': u'org.fedoraproject.prod.kerneltest.upload.new',
        u'msg': {
            u'test': {
                u'testset': u'default',
                u'authenticated': True,
                u'tester': u'jforbes',
                u'kernel_version': u'3.16.0-0.rc3.git3.1.fc21.x86_64',
                u'testdate': u'Mon Jul  7 11:15:23 EDT 2014',
                u'result': u'PASS',
                u'failed_tests': u'None',
                u'release': u'Fedora release 21 (Rawhide)',
                u'fedora_version': u'21',
                u'arch': u'x86_64',
            },
            u'agent': u'jforbes',
        }
    }


class TestKerneltestNewRelease(Base):
    """ As part of the `kernel testing initiative
    <https://fedoraproject.org/wiki/KernelTestingInitiative>`_, we built an
    `app <https://apps.fedoraproject.org/kerneltest>`_ that lets both real
    users and automated systems provide test feedback on new kernel builds and
    releases.

    *These* kinds of messages are published whenever an administrator sets up
    a new release for testing.
    """
    expected_title = "kerneltest.release.new"
    expected_subti = "jforbes added a new release for kerneltest: 21 TEST"
    expected_link = "https://apps.fedoraproject.org/kerneltest/stats"
    expected_icon = "https://apps.fedoraproject.org/img/icons/tux.png"
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "03ead14065750aad5e4eb5ee42f7c60fb1e50ea2f5ba2dc9a89639bc381796da"
        "?s=64&d=retro")
    expected_packages = set(['kernel'])
    expected_usernames = set(['jforbes'])
    expected_objects = set([
        'release/21/TEST',
    ])
    msg = {
        u'username': u'apache',
        u'i': 1,
        u'timestamp': 1404746131,
        u'msg_id': u'2014-4c25f684-c965-4003-b22d-57cda0e2e12c',
        u'crypto': u'x509',
        u'topic': u'org.fedoraproject.prod.kerneltest.release.new',
        u'msg': {
            u'release': {
                u'releasenum': 21,
                u'support': 'TEST',
            },
            u'agent': u'jforbes',
        }
    }


class TestKerneltestEditRelease(Base):
    """ As part of the `kernel testing initiative
    <https://fedoraproject.org/wiki/KernelTestingInitiative>`_, we built an
    `app <https://apps.fedoraproject.org/kerneltest>`_ that lets both real
    users and automated systems provide test feedback on new kernel builds and
    releases.

    *These* kinds of messages are published whenever an administrator edits an
    existing release against which tests can be run.
    """
    expected_title = "kerneltest.release.edit"
    expected_subti = "jforbes edited the 21-TEST release for kerneltest"
    expected_link = "https://apps.fedoraproject.org/kerneltest/stats"
    expected_icon = "https://apps.fedoraproject.org/img/icons/tux.png"
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "03ead14065750aad5e4eb5ee42f7c60fb1e50ea2f5ba2dc9a89639bc381796da"
        "?s=64&d=retro")
    expected_packages = set(['kernel'])
    expected_usernames = set(['jforbes'])
    expected_objects = set([
        'release/21/TEST',
    ])
    msg = {
        u'username': u'apache',
        u'i': 1,
        u'timestamp': 1404746131,
        u'msg_id': u'2014-4c25f684-c965-4003-b22d-57cda0e2e12c',
        u'crypto': u'x509',
        u'topic': u'org.fedoraproject.prod.kerneltest.release.edit',
        u'msg': {
            u'release': {
                u'releasenum': 21,
                u'support': 'TEST',
            },
            u'agent': u'jforbes',
        }
    }


add_doc(locals())

if __name__ == '__main__':
    unittest.main()



# This file is part of fedmsg.
# Copyright (C) 2015 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Pierre-Yves Chibon <pingou@pingoured.fr>
#           Ralph Bean <rbean@redhat.com>
#

from fedmsg_meta_fedora_infrastructure import BaseProcessor

import fedmsg.meta.base


def get_packages(message):
    ''' Returns the list of all the packages mentionned in the message. '''
    if 'differences' in message['msg']:
        return _get_packages_oldschool(message)
    else:
        return _get_packages_newschool(message)

def _get_packages_oldschool(message):
    pkgs = set()
    for category in message['msg']['differences']:
        for action in message['msg']['differences'][category]:
            for details in message['msg']['differences'][category][action]:
                if '/' in details[0]:
                    name = details[-1]
                else:
                    name = details[0]
                name = name.split('(')[0]
                pkgs.add(name)
    return pkgs


def _get_packages_newschool(message):
    return message['msg']['packages']


def get_objects(message):
    if 'differences' in message['msg']:
        return _get_objects_oldschool(message)
    else:
        return _get_objects_newschool(message)

def _get_objects_oldschool(message):
    repo = message['msg']['name']
    for category in message['msg']['differences']:
        for action in message['msg']['differences'][category]:
            for details in message['msg']['differences'][category][action]:
                if '/' in details[0]:
                    name = details[-1]
                else:
                    name = details[0]
                yield '%s/%s/%s/%s' % (repo, category, action, name)


def _get_objects_newschool(message):
    repo = message['msg']['name']
    for name in message['msg']['packages']:
        yield '%s/%s' % (repo, name)


def get_summary(message):
    ''' Returns a summary of the addition/deletion for each category in
    the specified message.
    '''
    if 'differences' in message['msg']:
        return ', '.join(_get_summary_oldschool(message))
    else:
        return _get_summary_newschool(message)

def _get_summary_oldschool(message):
    summary = list()
    for category in sorted(message['msg']['differences']):
        count_a = len(message['msg']['differences'][category]['added'])
        count_b = len(message['msg']['differences'][category]['removed'])
        if not count_a and not count_b:
            continue
        summary.append('{0}: +{1}/-{2}'.format(category, count_a, count_b))
    return summary

def _get_summary_newschool(message):
    packages = message['msg']['packages']
    return fedmsg.meta.base.BaseConglomerator.list_to_series(packages)

class MdapiProcessor(BaseProcessor):
    __name__ = "mdapi"
    __description__ = "the Fedora repository meta-data API"
    __link__ = "https://apps.fedoraproject.org/mdapi"
    __docs__ = "https://apps.fedoraproject.org/mdapi"
    __obj__ = "Medata API update"
    __icon__ = ("https://apps.fedoraproject.org/packages/"
                "images/icons/package_128x128.png")

    def subtitle(self, msg, **config):
        if 'mdapi.repo.update' in msg['topic']:
            tmpl = self._(
                u"mdapi noticed a {repo} repomd change: {summary}"
            )
            repo = msg['msg']['name']
            summary = get_summary(msg)
            return tmpl.format(repo=repo, summary=summary)
        else:
            raise NotImplementedError("%r" % msg)

    def secondary_icon(self, msg, **config):
        return self.__icon__

    def link(self, msg, **config):
        url = msg['msg']['url']
        if url.startswith('http'):
            return url
        else:
            return 'https://download.fedoraproject.org/pub/' + url

    def objects(self, msg, **config):
        return set(get_objects(msg))

    def packages(self, msg, **config):
        return set(get_packages(msg))



# This file is part of fedmsg.
# Copyright (C) 2014 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>

from fedmsg_meta_fedora_infrastructure import BaseProcessor


class HotnessProcessor(BaseProcessor):
    __name__ = 'hotness'
    __description__ = "A fedmsg consumer that files bugzilla bugs " + \
        "for upstream releases"
    __link__ = "https://release-monitoring.org"
    __docs__ = "https://fedoraproject.org/wiki/Upstream_release_monitoring"
    __obj__ = "Fresh Builds & Bugs"
    __icon__ = ("https://apps.fedoraproject.org/packages/"
                "images/icons/package_128x128.png")

    def subtitle(self, msg, **config):
        if 'hotness.update.bug.file' in msg['topic']:
            original = msg['msg']['trigger']['msg']
            if msg['msg']['trigger']['topic'].endswith('.project.map.new'):
                packages = [original['message']['new']]
            else:
                packages = [
                    pkg['package_name']
                    for pkg in original['message']['packages']
                    if pkg['distro'] == 'Fedora'
                ]
            tmpl= self._('the-new-hotness filed a bug on {packages}')
            return tmpl.format(packages=", ".join(packages))
        elif 'hotness.update.bug.followup' in msg['topic']:
            # This could be one of two different kinds of followup
            original = msg['msg']['trigger']['msg']
            bug_id = msg['msg']['bug']['bug_id']
            if 'srpm' in original:
                srpm = original['srpm']

                # The original task statuses from koji can be found like this
                # >>> import koji
                # >>> koji.TASK_STATES.keys()
                # ['FAILED', 'FREE', 'ASSIGNED', 'CANCELED', 'CLOSED', 'OPEN']
                status_lookup = {
                    'FAILED': self._('failed'),
                    'FREE': self._('submitted'),
                    'ASSIGNED': self._('assigned'),
                    'CANCELLED': self._('cancelled'),
                    'CLOSED': self._('completed'),
                    'OPEN': self._('started'),
                }
                status = status_lookup.get(original['new'], 'unknown')

                tmpl= self._('scratch build of {srpm} '
                             'for RHBZ#{bug_id} {status}')
                return tmpl.format(srpm=srpm, bug_id=bug_id, status=status)
            else:
                user = original['owner']
                build = '-'.join([
                    original[k] for k in ['name', 'version', 'release']])

                # The original build statuses from koji can be found like this
                # >>> import koji
                # >>> koji.BUILD_STATES
                # {'BUILDING': 0, 'DELETED': 2, 'CANCELED': 4,
                # 'COMPLETE': 1, 'FAILED': 3}
                status_lookup = [
                    self._('started'),
                    self._('completed'),
                    self._('deleted'),
                    self._('cancelled'),
                    self._('failed'),
                ]
                status = status_lookup[original['new']]
                tmpl= self._("{user}'s real build of {build} "
                             "for RHBZ#{bug_id} {status}")
                return tmpl.format(user=user, build=build,
                                   bug_id=bug_id, status=status)
        elif 'hotness.update.drop' in msg['topic']:
            original = msg['msg']['trigger']['msg']
            reason = msg['msg']['reason']

            packages = self.packages(msg, **config)
            if packages:
                thing = list(packages)[0]
            else:
                thing = original['project']['name']

            prefix = self._('the-new-hotness saw an update for {thing}, but ')
            qualifiers = {
                'anitya': self._("release-monitoring.org doesn't know what "
                                 "that project is called in Fedora land"),
                'bugzilla': self._("a bugzilla ticket had already been filed"),
                'pkgdb': self._("pkgdb says the maintainers are not "
                                "interested in bugs being filed"),
                'rawhide': self._("no rawhide version of the "
                                  "package could be found yet")
            }
            errmsg = self._('.... I dunno.  Whatever.')
            return prefix.format(thing=thing) + qualifiers.get(reason, errmsg)
        elif 'hotness.project.map' in msg['topic']:
            original = msg['msg']['trigger']['msg']
            if 'package_listing' in original:
                package = original['package_listing']['package']['name']
            else:
                package = original['package']['name']

            if 'reason' in msg['msg']:
                return self._(
                    'hotness tried to map {package} to an upstream project, '
                    'but failed:  "{reason}"').format(
                        package=package,
                        reason=msg['msg']['reason'],
                    )
            elif 'total' in msg['msg']:
                return self._(
                    'hotness tried to map {package} to an upstream project, '
                    'but failed due to ambiguity.  {total} other projects '
                    'share the same homepage').format(
                        package=package,
                        total=msg['msg']['total']
                    )
            elif 'project' in msg['msg']:
                if msg['msg']['success']:
                    return self._(
                        'hotness mapped {package} to the pre-existing '
                        'upstream project {project}').format(
                            package=package,
                            project=msg['msg']['project']['name'],
                        )
                else:
                    return self._(
                        'hotness tried to map {package} to the pre-existing '
                        'upstream project {project}, but failed for unknown '
                        'reasons').format(
                            package=package,
                            project=msg['msg']['project']['name'],
                        )
            else:
                if msg['msg']['success']:
                    return self._(
                        'hotness mapped {package} to a '
                        'brand-new upstream project'
                    ).format(package=package)
                else:
                    return self._(
                        'hotness tried to map {package} to a brand-new '
                        'upstream project, but failed for unknown reasons'
                    ).format(package=package)

    def link(self, msg, **config):
        if 'bug' in msg['msg']:
            base = 'bugzilla.redhat.com/show_bug.cgi?id=%i'
            if '.stg.' in msg['topic']:
                base = 'partner-' + base
            return 'https://' + base % msg['msg']['bug']['bug_id']
        elif 'trigger' in msg['msg']:
            original = msg['msg']['trigger']['msg']
            if 'project' in original:
                base = 'https://release-monitoring.org/project/%i/'
                return base % original['project']['id']
            elif 'project' in msg['msg']:
                base = 'https://release-monitoring.org/project/%i/'
                return base % msg['msg']['project']['id']
            elif 'package_listing' in original:
                return original['package_listing']['package']['review_url']
            elif 'package' in original:
                return original['package']['upstream_url']

    def secondary_icon(self, msg, **config):
        tmpl = 'https://apps.fedoraproject.org/packages/images/icons/%s.png'
        packages = self.packages(msg, **config)
        return tmpl % (list(packages)[0] if packages else "package_128x128")

    def objects(self, msg, **config):
        bugs = []
        if 'bug' in msg['msg']:
            bugs = ['bugs/%i' % msg['msg']['bug']['bug_id']]

        if 'anitya' in msg['msg']['trigger']['topic']:
            original = msg['msg']['trigger']['msg']
            packages = [
                'packages/' + pkg['package_name']
                for pkg in original['message']['packages']
                if pkg['distro'] == 'Fedora'
            ]
            projects = ['projects/' + original['project']['name']]
            return set(packages + projects + bugs)

        if 'package_listing' in msg['msg']['trigger']['msg']:
            original = msg['msg']['trigger']['msg']
            packages = [
                'packages/' + original['package_listing']['package']['name']
            ]
            projects = []
            if 'project' in msg['msg']:
                projects += ['projects/' + msg['msg']['project']['name']]
            return set(packages + projects)

        if 'package' in msg['msg']['trigger']['msg']:
            original = msg['msg']['trigger']['msg']
            packages = [
                'packages/' + original['package']['name']
            ]
            return set(packages)

        return set(bugs)

    def usernames(self, msg, **config):
        if 'buildsys.build' in msg['msg']['trigger']['topic']:
            return set([msg['msg']['trigger']['msg']['owner']])

        return set()

    def packages(self, msg, **config):
        if 'anitya' in msg['msg']['trigger']['topic']:
            original = msg['msg']['trigger']['msg']

            packages = None
            if 'packages' in original['message']:
                packages = original['message']['packages']
            elif 'packages' in original:
                packages = original['packages']

            if packages:
                return set([
                    pkg['package_name'] for pkg in packages
                    if pkg['distro'] == 'Fedora'
                ])

        if 'package_listing' in msg['msg']['trigger']['msg']:
            original = msg['msg']['trigger']['msg']
            return set([original['package_listing']['package']['name']])

        if 'buildsys.build' in msg['msg']['trigger']['topic']:
            return set([msg['msg']['trigger']['msg']['name']])

        if 'package' in msg['msg']['trigger']['msg']:
            original = msg['msg']['trigger']['msg']
            return set([original['package']['name']])

        return set()



# This file is part of fedmsg.
# Copyright (C) 2016 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Adam Williamson <awilliam@redhat.com>
#
"""Tests for openQA messages."""

import unittest
from fedmsg.tests.test_meta import Base
from .common import add_doc

class TestOpenQAJobDuplicateAuto(Base):
    """openQA emits messages on this topic when a job is duplicated.
    The 'id' is the job that was duplicated, the 'result' is the new
    job.
    """
    expected_title = "openqa.job.duplicate"
    expected_subti = (
        "job 10625 (test base_selinux on 64bit for iso "
        "Fedora-Everything-netinst-x86_64-24-20160323.n.0.iso) automatically "
        "duplicated as 10695 for Fedora-24-20160323.n.0")
    expected_link = "https://openqa.fedoraproject.org/tests/10625"
    expected_objects = set(
        ["Fedora-24-20160323.n.0",
         "Fedora-Everything-netinst-x86_64-24-20160323.n.0.iso"])
    msg = {
        "i": 1,
        "msg": {
            "ARCH": "x86_64",
            "BUILD": "Fedora-24-20160323.n.0",
            "FLAVOR": "Everything-boot-iso",
            "ISO": "Fedora-Everything-netinst-x86_64-24-20160323.n.0.iso",
            "MACHINE": "64bit",
            "TEST": "base_selinux",
            "auto": "1",
            "id": 10625,
            "remaining": 39,
            "result": "10695"
        },
        "msg_id": "2016-809a50c2-0829-46b7-beb1-e9cf0ed2ca1a",
        "source_name": "datanommer",
        "source_version": "0.6.5",
        "timestamp": 1458740621.0,
        "topic": "org.fedoraproject.prod.openqa.job.duplicate"
    }


class TestOpenQAJobDuplicateManual(Base):
    """openQA emits messages on this topic when a job is duplicated.
    The 'id' is the job that was duplicated, the 'result' is the new
    job.
    """
    # here we're testing the 'manual' vs. 'auto' difference
    expected_title = "openqa.job.duplicate"
    expected_subti = (
        "job 10625 (test base_selinux on 64bit for iso "
        "Fedora-Everything-netinst-x86_64-24-20160323.n.0.iso) manually "
        "duplicated as 10695 for Fedora-24-20160323.n.0")
    expected_link = "https://openqa.fedoraproject.org/tests/10625"
    expected_objects = set(
        ["Fedora-24-20160323.n.0",
         "Fedora-Everything-netinst-x86_64-24-20160323.n.0.iso"])
    # note this test also checks behaviour when both ISO and HDD_1
    # are present vs. only one or the other. Should behave exactly
    # like when only ISO is present, as when both are present, HDD_1
    # is usually a test asset that is of no external interest.
    msg = {
        "i": 1,
        "msg": {
            "ARCH": "x86_64",
            "BUILD": "Fedora-24-20160323.n.0",
            "FLAVOR": "Everything-boot-iso",
            "HDD_1": "disk_Everything-boot-iso_64bit.qcow2",
            "ISO": "Fedora-Everything-netinst-x86_64-24-20160323.n.0.iso",
            "MACHINE": "64bit",
            "TEST": "base_selinux",
            "auto": "0",
            "id": 10625,
            "remaining": 39,
            "result": "10695"
        },
        "msg_id": "2016-809a50c2-0829-46b7-beb1-e9cf0ed2ca1a",
        "source_name": "datanommer",
        "source_version": "0.6.5",
        "timestamp": 1458740621.0,
        "topic": "org.fedoraproject.prod.openqa.job.duplicate"
    }


class TestOpenQAJobRestart(Base):
    """openQA emits messages on this topic when a job is restarted.
    The 'id' is the job that was restarted, the 'result' is the id of
    the new job.
    """
    expected_title = "openqa.job.restart"
    expected_subti = (
        "job 10589 (test server_simple_encrypted on uefi for iso "
        "Fedora-Everything-netinst-x86_64-24_Alpha-1.7.iso) restarted as "
        "10619 for Fedora-24-20160322.4")
    expected_link = "https://openqa.fedoraproject.org/tests/10589"
    expected_objects = set(
        ["Fedora-24-20160322.4",
         "Fedora-Everything-netinst-x86_64-24_Alpha-1.7.iso"])
    msg = {
        "i": 1,
        "msg": {
            "ARCH": "x86_64",
            "BUILD": "Fedora-24-20160322.4",
            "FLAVOR": "universal",
            "ISO": "Fedora-Everything-netinst-x86_64-24_Alpha-1.7.iso",
            "MACHINE": "uefi",
            "TEST": "server_simple_encrypted",
            "id": 10589,
            "remaining": 2,
            "result": "10619"
        },
        "msg_id": "2016-02ec7b3a-ca9d-4844-8361-f46bc883a91d",
        "source_name": "datanommer",
        "source_version": "0.6.5",
        "timestamp": 1458702088.0,
        "topic": "org.fedoraproject.prod.openqa.job.restart"
    }

class TestOpenQAJobDoneStaging(Base):
    """openQA emits messages on this topic when a job completes.
    'remaining' indicates the number of jobs for the same compose that
    are either running or waiting to start. 'result' should indicate
    the result of the job, but at present it is always None due to
    upstream openQA issues.
    """
    expected_title = "openqa.job.done"
    expected_subti = (
        "staging job 10826 (test install_xfs on 64bit for iso "
        "Fedora-Everything-netinst-x86_64-Rawhide-20160323.n.0.iso) completed "
        "for Fedora-Rawhide-20160323.n.0, 23 remaining jobs")
    expected_link = "https://openqa.stg.fedoraproject.org/tests/10826"
    expected_objects = set(
        ["Fedora-Rawhide-20160323.n.0",
         "Fedora-Everything-netinst-x86_64-Rawhide-20160323.n.0.iso"])
    msg = {
        "i": 1,
        "msg": {
            "ARCH": "x86_64",
            "BUILD": "Fedora-Rawhide-20160323.n.0",
            "FLAVOR": "universal",
            "ISO": "Fedora-Everything-netinst-x86_64-Rawhide-20160323.n.0.iso",
            "MACHINE": "64bit",
            "TEST": "install_xfs",
            "id": 10826,
            "newbuild": None,
            "remaining": 23,
            "result": None,
        },
        "msg_id": "2016-fb9690a0-96fd-41e7-a532-0547a4b9229b",
        "source_name": "datanommer",
        "source_version": "0.6.5",
        "timestamp": 1458779794.0,
        "topic": "org.fedoraproject.stg.openqa.job.done"
    }

class TestOpenQAHdd1(Base):
    """Here we check that a message with HDD_1 and no ISO behaves as
    expected (subtitle should refer to the disk image and it should be
    in the objects set). As I'm writing this we don't actually have
    any such messages, this is future-proofing for when we add ARM
    tests soon, so the message is faked up based on the previous test.
    """
    # This is a corner case behaviour test and we don't want the doc
    # string showing up in openqa.job.done docs.
    nodoc = True
    expected_title = "openqa.job.done"
    expected_subti = (
        "staging job 10826 (test base_selinux on arm for disk "
        "Fedora-Minimal-armhfp-Rawhide-20160323.0-sda.raw.xz) completed for "
        "Fedora-Rawhide-20160323.n.0, 23 remaining jobs")
    expected_link = "https://openqa.stg.fedoraproject.org/tests/10826"
    expected_objects = set(
        ["Fedora-Rawhide-20160323.n.0",
         "Fedora-Minimal-armhfp-Rawhide-20160323.0-sda.raw.xz"])
    msg = {
        "i": 1,
        "msg": {
            "ARCH": "armhfp",
            "BUILD": "Fedora-Rawhide-20160323.n.0",
            "FLAVOR": "universal",
            "HDD_1": "Fedora-Minimal-armhfp-Rawhide-20160323.0-sda.raw.xz",
            "MACHINE": "arm",
            "TEST": "base_selinux",
            "id": 10826,
            "newbuild": None,
            "remaining": 23,
            "result": None,
        },
        "msg_id": "2016-fb9690a0-96fd-41e7-a532-0547a4b9229b",
        "source_name": "datanommer",
        "source_version": "0.6.5",
        "timestamp": 1458779794.0,
        "topic": "org.fedoraproject.stg.openqa.job.done"
    }

# LEGACY tests: fedmsg format changed 2016-04 to include more data,
# meta messages were improved. These tests cover the old message
# format and expected meta bits. job IDs are reliably ints after the
# change, may have been unpredictably ints or strings before.

class TestLegacyOpenQAJobDuplicateAuto(Base):
    """openQA emits messages on this topic when a job is duplicated.
    The 'id' is the job that was duplicated, the 'result' is the new
    job.
    """
    expected_title = "openqa.job.duplicate"
    expected_subti = ("job 10625 automatically duplicated as 10695 for "
                      "Fedora-24-20160323.n.0")
    expected_link = "https://openqa.fedoraproject.org/tests/10625"
    expected_objects = set(["Fedora-24-20160323.n.0"])
    msg = {
        "i": 1,
        "msg": {
            "auto": "1",
            "build": "Fedora-24-20160323.n.0",
            "id": "10625",
            "remaining": 39,
            "result": "10695"
        },
        "msg_id": "2016-809a50c2-0829-46b7-beb1-e9cf0ed2ca1a",
        "source_name": "datanommer",
        "source_version": "0.6.5",
        "timestamp": 1458740621.0,
        "topic": "org.fedoraproject.prod.openqa.job.duplicate"
    }


class TestLegacyOpenQAJobDuplicateManual(Base):
    """openQA emits messages on this topic when a job is duplicated.
    The 'id' is the job that was duplicated, the 'result' is the new
    job.
    """
    # here we're testing the 'manual' vs. 'auto' difference
    expected_title = "openqa.job.duplicate"
    expected_subti = ("job 10625 manually duplicated as 10695 for "
                      "Fedora-24-20160323.n.0")
    expected_link = "https://openqa.fedoraproject.org/tests/10625"
    expected_objects = set(["Fedora-24-20160323.n.0"])
    msg = {
        "i": 1,
        "msg": {
            "auto": "0",
            "build": "Fedora-24-20160323.n.0",
            "id": "10625",
            "remaining": 39,
            "result": "10695"
        },
        "msg_id": "2016-809a50c2-0829-46b7-beb1-e9cf0ed2ca1a",
        "source_name": "datanommer",
        "source_version": "0.6.5",
        "timestamp": 1458740621.0,
        "topic": "org.fedoraproject.prod.openqa.job.duplicate"
    }


class TestLegacyOpenQAJobRestart(Base):
    """openQA emits messages on this topic when a job is restarted.
    The 'id' is the job that was restarted, the 'result' is the id of
    the new job.
    """
    expected_title = "openqa.job.restart"
    expected_subti = "job 10589 restarted as 10619 for Fedora-24-20160322.4"
    expected_link = "https://openqa.fedoraproject.org/tests/10589"
    expected_objects = set(["Fedora-24-20160322.4"])
    msg = {
        "i": 1,
        "msg": {
            "build": "Fedora-24-20160322.4",
            "id": "10589",
            "remaining": 2,
            "result": "10619"
        },
        "msg_id": "2016-02ec7b3a-ca9d-4844-8361-f46bc883a91d",
        "source_name": "datanommer",
        "source_version": "0.6.5",
        "timestamp": 1458702088.0,
        "topic": "org.fedoraproject.prod.openqa.job.restart"
    }

class TestLegacyOpenQAJobDoneStaging(Base):
    """openQA emits messages on this topic when a job completes.
    'remaining' indicates the number of jobs for the same compose that
    are either running or waiting to start. 'result' should indicate
    the result of the job, but at present it is always None due to
    upstream openQA issues.
    """
    expected_title = "openqa.job.done"
    expected_subti = ("staging job 10826 completed for "
                      "Fedora-Rawhide-20160323.n.0, 23 remaining jobs")
    expected_link = "https://openqa.stg.fedoraproject.org/tests/10826"
    expected_objects = set(["Fedora-Rawhide-20160323.n.0"])
    msg = {
        "i": 1,
        "msg": {
            "build": "Fedora-Rawhide-20160323.n.0",
            "id": "10826",
            "newbuild": None,
            "remaining": 23,
            "result": None,
        },
        "msg_id": "2016-fb9690a0-96fd-41e7-a532-0547a4b9229b",
        "source_name": "datanommer",
        "source_version": "0.6.5",
        "timestamp": 1458779794.0,
        "topic": "org.fedoraproject.stg.openqa.job.done"
    }

add_doc(locals())

if __name__ == '__main__':
    unittest.main()



# This file is part of fedmsg.
# Copyright (C) 2015 Sayan Chowdhury.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Sayan Chowdhury <sayanchowdhury@fedoraproject.org>
#
""" Tests for autocloud messages """

import unittest

from fedmsg_meta_fedora_infrastructure.tests.base import Base

from .common import add_doc


class TestImageTestQueued(Base):
    """ These messages are published when an image is queued for testing in
    Autocloud app.
    """
    expected_title = "autocloud.image.queued"
    expected_subti = 'Fedora-Cloud-Base is queued for testing'
    expected_link = 'https://kojipkgs.fedoraproject.org//work/' + \
        'taskstasks/1410/10291410/Fedora-Cloud-Base-22-20150705.i386.qcow2'
    expected_secondary_icon = "https://apps.fedoraproject.org/img/icons/" + \
        "fedimg.png"
    expected_packages = set([])
    expected_usernames = set([])
    expected_objects = set(['autocloud/image/queued'])
    msg = {
        u'username': u'apache',
        u'i': 1,
        u'timestamp': 1379638157.759283,
        u'msg_id': u'2015-4b5aae66-b713-4c22-bb4a-1277d4402375',
        u'topic': u'org.fedoraproject.prod.autocloud.image.queued',
        u'msg': {
            u'buildid': u'10291410',
            u'status': u'queued',
            u'image_url': u'https://kojipkgs.fedoraproject.org//work/'
            'taskstasks/1410/10291410/Fedora-Cloud-Base-22-20150705.i386.qcow2',
            u'image_name': u'Fedora-Cloud-Base'
        }
    }


class TestImageTestRunning(Base):
    """ These messages are published when tests for an image starts
    in Autocloud app.
    """
    expected_title = "autocloud.image.running"
    expected_subti = 'The tests for the Fedora-Cloud-Base (rawhide) have started running'
    expected_link = 'https://kojipkgs.fedoraproject.org//work/' + \
        'taskstasks/1410/10291410/Fedora-Cloud-Base-22-20150705.i386.qcow2'
    expected_secondary_icon = "https://apps.fedoraproject.org/img/icons/" + \
        "fedimg.png"
    expected_packages = set([])
    expected_usernames = set([])
    expected_objects = set(['autocloud/image/running'])
    msg = {
        u'username': u'apache',
        u'i': 1,
        u'timestamp': 1379638157.759283,
        u'msg_id': u'2015-4b5aae66-b713-4c22-bb4a-1277d4402375',
        u'topic': u'org.fedoraproject.prod.autocloud.image.running',
        u'msg': {
            u'buildid': u'10291410',
            u'status': u'running',
            u'image_url': u'https://kojipkgs.fedoraproject.org//work/'
            'taskstasks/1410/10291410/Fedora-Cloud-Base-22-20150705.i386.qcow2',
            u'image_name': u'Fedora-Cloud-Base',
            u'release': 'rawhide',
        }
    }


class TestImageTestAborted(Base):
    """ These messages are published when tests for an image starts
    in Autocloud app.
    """
    expected_title = "autocloud.image.aborted"
    expected_subti = 'The tests for the Fedora-Cloud-Base (rawhide) have been aborted'
    expected_link = 'https://kojipkgs.fedoraproject.org//work/' + \
        'taskstasks/1410/10291410/Fedora-Cloud-Base-22-20150705.i386.qcow2'
    expected_secondary_icon = "https://apps.fedoraproject.org/img/icons/" + \
        "fedimg.png"
    expected_packages = set([])
    expected_usernames = set([])
    expected_objects = set(['autocloud/image/aborted'])
    msg = {
        u'username': u'apache',
        u'i': 1,
        u'timestamp': 1379638157.759283,
        u'msg_id': u'2015-4b5aae66-b713-4c22-bb4a-1277d4402375',
        u'topic': u'org.fedoraproject.prod.autocloud.image.aborted',
        u'msg': {
            u'buildid': u'10291410',
            u'status': u'aborted',
            u'image_url': u'https://kojipkgs.fedoraproject.org//work/'
            'taskstasks/1410/10291410/Fedora-Cloud-Base-22-20150705.i386.qcow2',
            u'image_name': u'Fedora-Cloud-Base',
            u'release': 'rawhide',
        }
    }


class TestImageTestFailed(Base):
    """ These messages are published when tests for an image starts
    in Autocloud app.
    """
    expected_title = "autocloud.image.failed"
    expected_subti = 'The tests for the Fedora-Cloud-Base (23) failed'
    expected_link = 'https://apps.fedoraproject.org/autocloud/jobs/412/output'
    expected_secondary_icon = "https://apps.fedoraproject.org/img/icons/" + \
        "fedimg.png"
    expected_packages = set([])
    expected_usernames = set([])
    expected_objects = set(['autocloud/image/failed'])
    msg = {
        u'username': u'apache',
        u'i': 1,
        u'timestamp': 1379638157.759283,
        u'msg_id': u'2015-4b5aae66-b713-4c22-bb4a-1277d4402375',
        u'topic': u'org.fedoraproject.prod.autocloud.image.failed',
        u'msg': {
            u'buildid': u'10291410',
            u'status': u'failed',
            u'job_id': 412,
            u'image_url': u'https://kojipkgs.fedoraproject.org//work/'
            'taskstasks/1410/10291410/Fedora-Cloud-Base-22-20150705.i386.qcow2',
            u'image_name': u'Fedora-Cloud-Base',
            u'release': u'23',
        }
    }


class TestImageTestSuccess(Base):
    """ These messages are published when tests for an image successfully
    completes in Autocloud app.
    """
    expected_title = "autocloud.image.success"
    expected_subti = 'The tests for Fedora-Cloud-Base were a success'
    expected_link = 'https://kojipkgs.fedoraproject.org//work/' + \
        'taskstasks/1410/10291410/Fedora-Cloud-Base-22-20150705.i386.qcow2'
    expected_secondary_icon = "https://apps.fedoraproject.org/img/icons/" + \
        "fedimg.png"
    expected_packages = set([])
    expected_usernames = set([])
    expected_objects = set(['autocloud/image/success'])
    msg = {
        u'username': u'apache',
        u'i': 1,
        u'timestamp': 1379638157.759283,
        u'msg_id': u'2015-4b5aae66-b713-4c22-bb4a-1277d4402375',
        u'topic': u'org.fedoraproject.prod.autocloud.image.success',
        u'msg': {
            u'buildid': u'10291410',
            u'status': u'success',
            u'image_url': u'https://kojipkgs.fedoraproject.org//work/'
            'taskstasks/1410/10291410/Fedora-Cloud-Base-22-20150705.i386.qcow2',
            u'image_name': u'Fedora-Cloud-Base'
        }
    }


add_doc(locals())

if __name__ == '__main__':
    unittest.main()



# This file is part of fedmsg.
# Copyright (C) 2016 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Adam Williamson <awilliam@redhat.com>

from fedmsg_meta_fedora_infrastructure import BaseProcessor

class OpenQAProcessor(BaseProcessor):
    __name__ = "openqa"
    __description__ = "Distribution-level automated testing"
    __link__ = "https://openqa.fedoraproject.org"
    __docs__ = "https://os-autoinst.github.io/openQA/documentation/"
    __obj__ = "openQA test results"
    # there is no distro-neutral openQA icon, they use the SUSE geeko
    #__icon__ = "https://apps.fedoraproject.org/img/icons/taskotron.png"

    def subtitle(self, msg, **config):
        job = msg['msg'].get('id', '')
        # old messages had 'build' not 'BUILD'
        build = msg['msg'].get('BUILD', msg['msg'].get('build', ''))
        remain = msg['msg'].get('remaining', '')
        result = msg['msg'].get('result')
        # all the following do not exist in old messages so we always
        # check before using them
        test = msg['msg'].get('TEST')
        arch = msg['msg'].get('ARCH')
        machine = msg['msg'].get('MACHINE')
        flavor = msg['msg'].get('FLAVOR')
        iso = msg['msg'].get('ISO')
        hdd1 = msg['msg'].get('HDD_1')

        msgtmpl = ''
        if ".stg" in msg['topic']:
            msgtmpl = "staging "

        msgtmpl += "job {0} ".format(job)
        if test:
            # if we have 'test' we can assume 'machine'
            msgtmpl += "(test {0} on {1}".format(test, machine)
            if iso:
                msgtmpl += " for iso {0}) ".format(iso)
            elif hdd1:
                msgtmpl += " for disk {0}) ".format(hdd1)
            else:
                # just for general safety...
                msgtmpl += ") "

        if msg['topic'].endswith('job.duplicate'):
            auto = msg['msg'].get('auto', '')
            if auto == "0":
                auto = "manually "
            if auto == "1":
                auto = "automatically "
            msgtmpl += "{0}duplicated as {1} for {2}"
            return msgtmpl.format(auto, result, build)

        if msg['topic'].endswith('job.restart'):
            msgtmpl += "restarted as {0} for {1}"
            return msgtmpl.format(result, build)

        if msg['topic'].endswith('job.done'):
            msgtmpl += "completed for {0}, {1} remaining jobs"
            return msgtmpl.format(build, remain)

    def link(self, msg, **config):
        urltmpl = 'https://openqa.fedoraproject.org/tests/{0}'
        if '.stg' in msg['topic']:
            urltmpl = 'https://openqa.stg.fedoraproject.org/tests/{0}'
        return urltmpl.format(msg['msg'].get('id'))

    def objects(self, msg, **config):
        objs = []
        if 'build' in msg['msg']:
            objs.append(msg['msg']['build'])
        elif 'BUILD' in msg['msg']:
            objs.append(msg['msg']['BUILD'])

        if 'ISO' in msg['msg']:
            objs.append(msg['msg']['ISO'])
        # HDD_1 is probably not an 'object' if there's an ISO, it's
        # more likely just a test component
        elif 'HDD_1' in msg['msg']:
            objs.append(msg['msg']['HDD_1'])

        return set(objs)



# This file is part of fedmsg.
# Copyright (C) 2013 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
""" Tests for compose messages """

from fedmsg.tests.test_meta import Base

from .common import add_doc


class TestLegacyComposeBranchedComplete(Base):
    """ This tests "old school" compose messages.

    :mod:`fedmsg.meta` needs to be able to handle them because they are
    stored, forever, in datanommer.
    """
    expected_title = "compose.branched.complete"
    expected_subti = "f18 compose completed"
    expected_link = \
        "https://dl.fedoraproject.org/pub/fedora/linux/development/f18"
    expected_objects = set(['branched/primary'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.branched.complete",
        "msg": {
            "log": "done",
            "branch": "f18",
        },
    }


class TestLegacyComposeBranchedStart(Base):
    """ This tests "old school" compose messages.

    :mod:`fedmsg.meta` needs to be able to handle them because they are
    stored, forever, in datanommer.
    """
    expected_title = "compose.branched.start"
    expected_subti = "f18 compose started"
    expected_objects = set(['branched/primary'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.branched.start",
        "msg": {
            "log": "start",
            "branch": "f18",
        },
    }


class TestLegacyComposeBranchedMashStart(Base):
    """ This tests "old school" compose messages.

    :mod:`fedmsg.meta` needs to be able to handle them because they are
    stored, forever, in datanommer.
    """
    expected_title = "compose.branched.mash.start"
    expected_subti = "f18 compose started mashing"
    expected_objects = set(['branched/primary'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.branched.mash.start",
        "msg": {
            "log": "start",
            "branch": "f18",
        },
    }


class TestLegacyComposeBranchedMashComplete(Base):
    """ This tests "old school" compose messages.

    :mod:`fedmsg.meta` needs to be able to handle them because they are
    stored, forever, in datanommer.
    """
    expected_title = "compose.branched.mash.complete"
    expected_subti = "f18 compose finished mashing"
    expected_objects = set(['branched/primary'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.branched.mash.complete",
        "msg": {
            "log": "done",
            "branch": "f18",
        },
    }


class TestLegacyComposeBranchedPungifyStart(Base):
    """ This tests "old school" compose messages.

    :mod:`fedmsg.meta` needs to be able to handle them because they are
    stored, forever, in datanommer.
    """
    expected_title = "compose.branched.pungify.start"
    expected_subti = "started building boot.iso for f18"
    expected_objects = set(['branched/primary'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.branched.pungify.start",
        "msg": {
            "log": "start",
            "branch": "f18",
        },
    }


class TestLegacyComposeBranchedPungifyComplete(Base):
    """ This tests "old school" compose messages.

    :mod:`fedmsg.meta` needs to be able to handle them because they are
    stored, forever, in datanommer.
    """
    expected_title = "compose.branched.pungify.complete"
    expected_subti = "finished building boot.iso for f18"
    expected_objects = set(['branched/primary'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.branched.pungify.complete",
        "msg": {
            "log": "done",
            "branch": "f18",
        },
    }


class TestLegacyComposeBranchedRsyncStart(Base):
    """ This tests "old school" compose messages.

    :mod:`fedmsg.meta` needs to be able to handle them because they are
    stored, forever, in datanommer.
    """
    expected_title = "compose.branched.rsync.start"
    expected_subti = "started rsyncing f18 compose"
    expected_objects = set(['branched/primary'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.branched.rsync.start",
        "msg": {
            "log": "start",
            "branch": "f18",
        },
    }


class TestLegacyComposeBranchedRsyncComplete(Base):
    """ This tests "old school" compose messages.

    :mod:`fedmsg.meta` needs to be able to handle them because they are
    stored, forever, in datanommer.
    """
    expected_title = "compose.branched.rsync.complete"
    expected_subti = \
        "finished rsync of f18 compose"
    expected_link = \
        "https://dl.fedoraproject.org/pub/fedora/linux/development/f18"
    expected_objects = set(['branched/primary'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.branched.rsync.complete",
        "msg": {
            "log": "done",
            "branch": "f18",
        },
    }


class TestLegacyComposeRawhideComplete(Base):
    """ This tests "old school" compose messages.

    :mod:`fedmsg.meta` needs to be able to handle them because they are
    stored, forever, in datanommer.
    """
    expected_title = "compose.rawhide.complete"
    expected_subti = "rawhide compose completed"
    expected_link = \
        "https://dl.fedoraproject.org/pub/fedora/linux/development/rawhide"
    expected_objects = set(['rawhide/primary'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.rawhide.complete",
        "msg": {
            "log": "done",
            "branch": "rawhide",
        },
    }


class TestLegacyComposeRawhideStart(Base):
    """ This tests "old school" compose messages.

    :mod:`fedmsg.meta` needs to be able to handle them because they are
    stored, forever, in datanommer.
    """
    expected_title = "compose.rawhide.start"
    expected_subti = "rawhide compose started"
    expected_objects = set(['rawhide/primary'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.rawhide.start",
        "msg": {
            "log": "start",
            "branch": "rawhide",
        },
    }


class TestLegacyComposeRawhideMashStart(Base):
    """ This tests "old school" compose messages.

    :mod:`fedmsg.meta` needs to be able to handle them because they are
    stored, forever, in datanommer.
    """
    expected_title = "compose.rawhide.mash.start"
    expected_subti = "rawhide compose started mashing"
    expected_objects = set(['rawhide/primary'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.rawhide.mash.start",
        "msg": {
            "log": "start",
            "branch": "rawhide",
        },
    }


class TestLegacyComposeRawhideMashComplete(Base):
    """ This tests "old school" compose messages.

    :mod:`fedmsg.meta` needs to be able to handle them because they are
    stored, forever, in datanommer.
    """
    expected_title = "compose.rawhide.mash.complete"
    expected_subti = "rawhide compose finished mashing"
    expected_objects = set(['rawhide/primary'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.rawhide.mash.complete",
        "msg": {
            "log": "done",
            "branch": "rawhide",
        },
    }


class TestLegacyComposeRawhideRsyncStart(Base):
    """ This tests "old school" compose messages.

    :mod:`fedmsg.meta` needs to be able to handle them because they are
    stored, forever, in datanommer.
    """
    expected_title = "compose.rawhide.rsync.start"
    expected_subti = "started rsyncing rawhide compose"
    expected_objects = set(['rawhide/primary'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.rawhide.rsync.start",
        "msg": {
            "log": "start",
            "branch": "rawhide",
        },
    }


class TestLegacyComposeRawhideRsyncComplete(Base):
    """ This tests "old school" compose messages.

    :mod:`fedmsg.meta` needs to be able to handle them because they are
    stored, forever, in datanommer.
    """
    expected_title = "compose.rawhide.rsync.complete"
    expected_subti = "finished rsync of rawhide compose"
    expected_link = \
        "https://dl.fedoraproject.org/pub/fedora/linux/development/rawhide"
    expected_objects = set(['rawhide/primary'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.rawhide.rsync.complete",
        "msg": {
            "log": "done",
            "branch": "rawhide",
        },
    }


class TestComposeBranchedComplete(Base):
    """ The `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ "compose" scripts
    produce these messages when they have
    **finished composing**
    whatever the current branched distribution version is.  They are published
    for both primary and secondary architectures.  The example here is of a
    **primary** arch message (the empty string signifies primary).
    """
    expected_title = "compose.branched.complete"
    expected_subti = "f18 compose completed"
    expected_link = \
        "https://dl.fedoraproject.org/pub/fedora/linux/development/f18"
    expected_objects = set(['branched/primary'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.branched.complete",
        "msg": {
            "log": "done",
            "branch": "f18",
            "arch": "",
        },
    }


class TestComposeEPELBetaComplete(Base):
    """ The `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ "compose" scripts
    produce these messages when they have **finished composing** the EPEL beta.
    """
    expected_title = "compose.epelbeta.complete"
    expected_subti = "epelbeta compose completed"
    expected_link = "https://dl.fedoraproject.org/pub/epel/beta/7/"
    expected_objects = set(['epelbeta/primary'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.epelbeta.complete",
        "msg": {
            "log": "done",
        },
    }


class TestComposeBranchedStart(Base):
    """ The `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ "compose" scripts
    produce these messages when they have
    **begun composing**
    whatever the current branched distribution version is.  They are published
    for both primary and secondary architectures.  The example here is of a
    **primary** arch message (the empty string signifies primary).
    """
    expected_title = "compose.branched.start"
    expected_subti = "f18 compose started"
    expected_objects = set(['branched/primary'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.branched.start",
        "msg": {
            "log": "start",
            "branch": "f18",
            "arch": "",
        },
    }


class TestComposeBranchedMashStart(Base):
    """ The `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ "compose" scripts
    produce these messages when they have
    **begun mashing** for
    whatever the current branched distribution version is.  They are published
    for both primary and secondary architectures.  The example here is of a
    **primary** arch message (the empty string signifies primary).
    """
    expected_title = "compose.branched.mash.start"
    expected_subti = "f18 compose started mashing"
    expected_objects = set(['branched/primary'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.branched.mash.start",
        "msg": {
            "log": "start",
            "branch": "f18",
            "arch": "",
        },
    }


class TestComposeBranchedMashComplete(Base):
    """ The `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ "compose" scripts
    produce these messages when they have
    **finished mashing**
    whatever the current branched distribution version is.  They are published
    for both primary and secondary architectures.  The example here is of a
    **primary** arch message (the empty string signifies primary).
    """
    expected_title = "compose.branched.mash.complete"
    expected_subti = "f18 compose finished mashing"
    expected_objects = set(['branched/primary'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.branched.mash.complete",
        "msg": {
            "log": "done",
            "branch": "f18",
            "arch": "",
        },
    }


class TestComposeBranchedPungifyStart(Base):
    """ The `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ "compose" scripts
    produce these messages when they have
    started the `pungify <https://fedorahosted.org/pungi/>`_ process for
    whatever the current branched distribution version is.  They are published
    for both primary and secondary architectures.  The example here is of a
    **primary** arch message (the empty string signifies primary).
    """
    expected_title = "compose.branched.pungify.start"
    expected_subti = "started building boot.iso for f18"
    expected_objects = set(['branched/primary'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.branched.pungify.start",
        "msg": {
            "log": "start",
            "branch": "f18",
            "arch": "",
        },
    }


class TestComposeBranchedPungifyComplete(Base):
    """ The `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ "compose" scripts
    produce these messages when they have
    completed the `pungify <https://fedorahosted.org/pungi/>`_ process for
    whatever the current branched distribution version is.  They are published
    for both primary and secondary architectures.  The example here is of a
    **primary** arch message (the empty string signifies primary).
    """
    expected_title = "compose.branched.pungify.complete"
    expected_subti = "finished building boot.iso for f18"
    expected_objects = set(['branched/primary'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.branched.pungify.complete",
        "msg": {
            "log": "done",
            "branch": "f18",
            "arch": "",
        },
    }


class TestComposeBranchedRsyncStart(Base):
    """ The `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ "compose" scripts
    produce these messages when they have
    begun **rsyncing**
    whatever the current branched distribution version is.  They are published
    for both primary and secondary architectures.  The example here is of a
    **primary** arch message (the empty string signifies primary).
    """
    expected_title = "compose.branched.rsync.start"
    expected_subti = "started rsyncing f18 compose"
    expected_objects = set(['branched/primary'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.branched.rsync.start",
        "msg": {
            "log": "start",
            "branch": "f18",
            "arch": "",
        },
    }


class TestComposeBranchedRsyncComplete(Base):
    """ The `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ "compose" scripts
    produce these messages when they have
    finished **rsyncing**
    whatever the current branched distribution version is.  They are published
    for both primary and secondary architectures.  The example here is of a
    **primary** arch message (the empty string signifies primary).
    """
    expected_title = "compose.branched.rsync.complete"
    expected_subti = \
        "finished rsync of f18 compose"
    expected_link = \
        "https://dl.fedoraproject.org/pub/fedora/linux/development/f18"
    expected_objects = set(['branched/primary'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.branched.rsync.complete",
        "msg": {
            "log": "done",
            "branch": "f18",
            "arch": "",
        },
    }


class TestComposeRawhideComplete(Base):
    """ The `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ "compose" scripts
    produce these messages when they have
    **finished** the rawhide compose.  They are published
    for both primary and secondary architectures.  The example here is of a
    **primary** arch message (the empty string signifies primary).
    """
    expected_title = "compose.rawhide.complete"
    expected_subti = "rawhide compose completed"
    expected_objects = set(['rawhide/primary'])
    expected_link = \
        "https://dl.fedoraproject.org/pub/fedora/linux/development/rawhide"
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.rawhide.complete",
        "msg": {
            "log": "done",
            "branch": "rawhide",
            "arch": "",
        },
    }


class TestComposeRawhideStart(Base):
    """ The `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ "compose" scripts
    produce these messages when they have
    **started** the rawhide compose.  They are published
    for both primary and secondary architectures.  The example here is of a
    **primary** arch message (the empty string signifies primary).
    """
    expected_title = "compose.rawhide.start"
    expected_subti = "rawhide compose started"
    expected_objects = set(['rawhide/primary'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.rawhide.start",
        "msg": {
            "log": "start",
            "branch": "rawhide",
            "arch": "",
        },
    }


class TestComposeRawhideMashStart(Base):
    """ The `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ "compose" scripts
    produce these messages when they have
    **started mashing** the rawhide compose.  They are published
    for both primary and secondary architectures.  The example here is of a
    **primary** arch message (the empty string signifies primary).
    """
    expected_title = "compose.rawhide.mash.start"
    expected_subti = "rawhide compose started mashing"
    expected_objects = set(['rawhide/primary'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.rawhide.mash.start",
        "msg": {
            "log": "start",
            "branch": "rawhide",
            "arch": "",
        },
    }


class TestComposeRawhideMashComplete(Base):
    """ The `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ "compose" scripts
    produce these messages when they have
    **finished mashing** the rawhide compose.  They are published
    for both primary and secondary architectures.  The example here is of a
    **primary** arch message (the empty string signifies primary).
    """
    expected_title = "compose.rawhide.mash.complete"
    expected_subti = "rawhide compose finished mashing"
    expected_objects = set(['rawhide/primary'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.rawhide.mash.complete",
        "msg": {
            "log": "done",
            "branch": "rawhide",
            "arch": "",
        },
    }


class TestComposeRawhideRsyncStart(Base):
    """ The `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ "compose" scripts
    produce these messages when they have
    **started rsyncing** the rawhide compose.  They are published
    for both primary and secondary architectures.  The example here is of a
    **primary** arch message (the empty string signifies primary).
    """
    expected_title = "compose.rawhide.rsync.start"
    expected_subti = "started rsyncing rawhide compose"
    expected_objects = set(['rawhide/primary'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.rawhide.rsync.start",
        "msg": {
            "log": "start",
            "branch": "rawhide",
            "arch": "",
        },
    }


class TestComposeRawhideRsyncComplete(Base):
    """ The `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ "compose" scripts
    produce these messages when they have
    **finished rsyncing** the rawhide compose.  They are published
    for both primary and secondary architectures.  The example here is of a
    **primary** arch message (the empty string signifies primary).
    """
    expected_title = "compose.rawhide.rsync.complete"
    expected_subti = "finished rsync of rawhide compose"
    expected_link = \
        "https://dl.fedoraproject.org/pub/fedora/linux/development/rawhide"
    expected_objects = set(['rawhide/primary'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.rawhide.rsync.complete",
        "msg": {
            "log": "done",
            "branch": "rawhide",
            "arch": "",
        },
    }


class TestSecondaryArchComposeBranchedComplete(Base):
    """ The `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ "compose" scripts
    produce these messages when they have
    **finished composing**
    whatever the current branched distribution version is.  They are published
    for both primary and secondary architectures.  The example here is of a
    **secondary** arch message.
    """
    expected_title = "compose.branched.complete"
    expected_subti = "f18 compose (arm) completed"
    expected_link = \
        "https://dl.fedoraproject.org/pub/fedora-secondary/" + \
        "development/f18"
    expected_objects = set(['branched/arm'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.branched.complete",
        "msg": {
            "log": "done",
            "branch": "f18",
            "arch": "arm",
        },
    }


class TestSecondaryArchComposeBranchedStart(Base):
    """ The `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ "compose" scripts
    produce these messages when they have
    **started composing**
    whatever the current branched distribution version is.  They are published
    for both primary and secondary architectures.  The example here is of a
    **secondary** arch message.
    """
    expected_title = "compose.branched.start"
    expected_subti = "f18 compose (arm) started"
    expected_objects = set(['branched/arm'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.branched.start",
        "msg": {
            "log": "start",
            "branch": "f18",
            "arch": "arm",
        },
    }


class TestSecondaryArchComposeBranchedMashStart(Base):
    """ The `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ "compose" scripts
    produce these messages when they have
    **started mashing**
    whatever the current branched distribution version is.  They are published
    for both primary and secondary architectures.  The example here is of a
    **secondary** arch message.
    """
    expected_title = "compose.branched.mash.start"
    expected_subti = "f18 compose (arm) started mashing"
    expected_objects = set(['branched/arm'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.branched.mash.start",
        "msg": {
            "log": "start",
            "branch": "f18",
            "arch": "arm",
        },
    }


class TestSecondaryArchComposeBranchedMashComplete(Base):
    """ The `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ "compose" scripts
    produce these messages when they have
    **finished mashing**
    whatever the current branched distribution version is.  They are published
    for both primary and secondary architectures.  The example here is of a
    **secondary** arch message.
    """
    expected_title = "compose.branched.mash.complete"
    expected_subti = "f18 compose (arm) finished mashing"
    expected_objects = set(['branched/arm'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.branched.mash.complete",
        "msg": {
            "log": "done",
            "branch": "f18",
            "arch": "arm",
        },
    }


class TestSecondaryArchComposeBranchedPungifyStart(Base):
    """ The `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ "compose" scripts
    produce these messages when they have
    started the `pungify <https://fedorahosted.org/pungi/>`_ process for
    whatever the current branched distribution version is.  They are published
    for both primary and secondary architectures.  The example here is of a
    **secondary** arch message.
    """
    expected_title = "compose.branched.pungify.start"
    expected_subti = "started building boot.iso for f18 (arm)"
    expected_objects = set(['branched/arm'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.branched.pungify.start",
        "msg": {
            "log": "start",
            "branch": "f18",
            "arch": "arm",
        },
    }


class TestSecondaryArchComposeBranchedPungifyComplete(Base):
    """ The `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ "compose" scripts
    produce these messages when they have
    completed the `pungify <https://fedorahosted.org/pungi/>`_ process for
    whatever the current branched distribution version is.  They are published
    for both primary and secondary architectures.  The example here is of a
    **secondary** arch message.
    """
    expected_title = "compose.branched.pungify.complete"
    expected_subti = "finished building boot.iso for f18 (arm)"
    expected_objects = set(['branched/arm'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.branched.pungify.complete",
        "msg": {
            "log": "done",
            "branch": "f18",
            "arch": "arm",
        },
    }


class TestSecondaryArchComposeBranchedRsyncStart(Base):
    """ The `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ "compose" scripts
    produce these messages when they have
    **started rsyncing**
    whatever the current branched distribution version is.  They are published
    for both primary and secondary architectures.  The example here is of a
    **secondary** arch message.
    """
    expected_title = "compose.branched.rsync.start"
    expected_subti = "started rsyncing f18 compose (arm)"
    expected_objects = set(['branched/arm'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.branched.rsync.start",
        "msg": {
            "log": "start",
            "branch": "f18",
            "arch": "arm",
        },
    }


class TestSecondaryArchComposeBranchedRsyncComplete(Base):
    """ The `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ "compose" scripts
    produce these messages when they have
    **finished rsyncing**
    whatever the current branched distribution version is.  They are published
    for both primary and secondary architectures.  The example here is of a
    **secondary** arch message.
    """
    expected_title = "compose.branched.rsync.complete"
    expected_subti = \
        "finished rsync of f18 compose (arm)"
    expected_link = \
        "https://dl.fedoraproject.org/pub/fedora-secondary/" + \
        "development/f18"
    expected_objects = set(['branched/arm'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.branched.rsync.complete",
        "msg": {
            "log": "done",
            "branch": "f18",
            "arch": "arm",
        },
    }


class TestSecondaryArchComposeRawhideComplete(Base):
    """ The `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ "compose" scripts
    produce these messages when they have
    **finished** the rawhide compose.  They are published
    for both primary and secondary architectures.  The example here is of a
    **secondary** arch message.
    """
    expected_title = "compose.rawhide.complete"
    expected_subti = "rawhide compose (arm) completed"
    expected_link = \
        "https://dl.fedoraproject.org/pub/fedora-secondary/" + \
        "development/rawhide"
    expected_objects = set(['rawhide/arm'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.rawhide.complete",
        "msg": {
            "log": "done",
            "branch": "rawhide",
            "arch": "arm",
        },
    }


class TestSecondaryArchComposeRawhideStart(Base):
    """ The `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ "compose" scripts
    produce these messages when they have
    **started** the rawhide compose.  They are published
    for both primary and secondary architectures.  The example here is of a
    **secondary** arch message.
    """
    expected_title = "compose.rawhide.start"
    expected_subti = "rawhide compose (arm) started"
    expected_objects = set(['rawhide/arm'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.rawhide.start",
        "msg": {
            "log": "start",
            "branch": "rawhide",
            "arch": "arm",
        },
    }


class TestSecondaryArchComposeRawhideMashStart(Base):
    """ The `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ "compose" scripts
    produce these messages when they have
    **started mashing** the rawhide compose.  They are published
    for both primary and secondary architectures.  The example here is of a
    **secondary** arch message.
    """
    expected_title = "compose.rawhide.mash.start"
    expected_subti = "rawhide compose (arm) started mashing"
    expected_objects = set(['rawhide/arm'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.rawhide.mash.start",
        "msg": {
            "log": "start",
            "branch": "rawhide",
            "arch": "arm",
        },
    }


class TestSecondaryArchComposeRawhideMashComplete(Base):
    """ The `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ "compose" scripts
    produce these messages when they have
    **finished mashing** the rawhide compose.  They are published
    for both primary and secondary architectures.  The example here is of a
    **secondary** arch message.
    """
    expected_title = "compose.rawhide.mash.complete"
    expected_subti = "rawhide compose (arm) finished mashing"
    expected_objects = set(['rawhide/arm'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.rawhide.mash.complete",
        "msg": {
            "log": "done",
            "branch": "rawhide",
            "arch": "arm",
        },
    }


class TestSecondaryArchComposeRawhideRsyncStart(Base):
    """ The `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ "compose" scripts
    produce these messages when they have
    **started rsyncing** the rawhide compose.  They are published
    for both primary and secondary architectures.  The example here is of a
    **secondary** arch message.
    """
    expected_title = "compose.rawhide.rsync.start"
    expected_subti = "started rsyncing rawhide compose (arm)"
    expected_objects = set(['rawhide/arm'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.rawhide.rsync.start",
        "msg": {
            "log": "start",
            "branch": "rawhide",
            "arch": "arm",
        },
    }


class TestSecondaryArchComposeRawhideRsyncComplete(Base):
    """ The `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ "compose" scripts
    produce these messages when they have
    **finished rsyncing** the rawhide compose.  They are published
    for both primary and secondary architectures.  The example here is of a
    **secondary** arch message.
    """
    expected_title = "compose.rawhide.rsync.complete"
    expected_subti = \
        "finished rsync of rawhide compose (arm)"
    expected_link = \
        "https://dl.fedoraproject.org/pub/fedora-secondary/" + \
        "development/rawhide"
    expected_objects = set(['rawhide/arm'])
    msg = {
        "i": 1,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.compose.rawhide.rsync.complete",
        "msg": {
            "log": "done",
            "branch": "rawhide",
            "arch": "arm",
        },
    }


class TestMakeUpdatesStarted(Base):
    """ The `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ "compose" scripts
    produce these messages when they have **started** the make-updates process
    (which is how we do periodic re-spins of our cloud, atomic, and docker
    images).  Here's an example of a message from F23:
    """
    expected_title = "compose.23.make-updates.start"
    expected_subti = \
        "started a run of F23 make-updates"
    expected_link = \
        "https://dl.fedoraproject.org/pub/fedora/linux/releases/23"
    expected_objects = set(['23/primary'])
    msg = {
        "i": 1,
        "msg": {
            "branch": "23",
            "log": "start"
        },
        "timestamp": 1454303707.0,
        "topic": "org.fedoraproject.prod.compose.23.make-updates.start"
    }


add_doc(locals())



import fedmsg.meta.base
from fedmsg_meta_fedora_infrastructure.fasshim import avatar_url

class AbstractTaggerConglomerator(fedmsg.meta.base.BaseConglomerator):

    def merge(self, constituents, subject, **config):
        ms = constituents  # shorthand

        user = ms[0]['msg']['user']['username']
        tags = set([m['msg']['vote']['tag']['tag'] for m in ms])
        pkgs = set([m['msg']['vote']['tag']['package'] for m in ms])

        tags = self.list_to_series(tags)
        pkgs = self.list_to_series(pkgs)


        tmpl = self.produce_template(constituents, subject, **config)
        tmpl['subtitle'] = self.subtitle.format(user=user, tags=tags, pkgs=pkgs)
        tmpl['subjective'] = tmpl['subtitle']

        default = tmpl['icon']

        tmpl['secondary_icon'] = self.get_secondary_icon(constituents, default)
        tmpl['link'] = 'http://infrastructure.fedoraproject.org' + \
            '/infra/ansible.git/'

        return tmpl

    def matches(self, a, b, **config):
        """ The changes must all be **to** the same pkgdb user """
        a, b = a['msg'], b['msg']
        if a['user']['username'] != b['user']['username']:
            return False
        return True

    def get_secondary_icon(self, constituents, default):
        userid = constituents[0]['msg']['user']['username']
        return avatar_url(userid)

    def get_link(self, constituents):
        return ''


class UpdateByUser(AbstractTaggerConglomerator):
    subtitle = '{user} voted on the tags: {tags} to the packages: {pkgs}'

    def can_handle(self, msg, **config):
        return 'fedoratagger.tag.update' in msg['topic']


class CreateByUser(AbstractTaggerConglomerator):
    subtitle = '{user} created new tags: {tags} on the packages: {pkgs}'

    def can_handle(self, msg, **config):
        return 'fedoratagger.tag.create' in msg['topic']



# This file is part of fedmsg.
# Copyright (C) 2015 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
""" Tests for coprs messages """

import unittest

from fedmsg.tests.test_meta import Base

from .common import add_doc


class TestLegacyNagiosHostStateChange(Base):
    expected_title = "nagios.host.state.change"
    expected_subti = "proxy04.fedoraproject.org is down"
    expected_icon = 'https://apps.fedoraproject.org/img/icons/nagios-logo.png'
    expected_secondary_icon = expected_icon
    expected_packages = set([])
    expected_usernames = set([])
    expected_objects = set([
        'proxy04.fedoraproject.org/state',
    ])
    expected_link = "https://admin.fedoraproject.org/nagios/cgi-bin/" + \
        "status.cgi?navbarsearch=1&host=proxy04.fedoraproject.org"
    msg = {
        "i": 1,
        "timestamp": 1450127478.0,
        "msg_id": "2015-c29fd2e7-b4f3-4ac0-8eed-9aa2f6d0d675",
        "topic": "org.fedoraproject.prod.nagios.host.state.change",
        "msg": {
            "host": "proxy04.fedoraproject.org",
            "type": "PROBLEM",
            "state": "CRITICAL",
            "host_ack_author": "",
            "service_ack_author": "",
        }
    }


class TestLegacyNagiosServiceStateChange(Base):
    expected_title = "nagios.service.state.change"
    expected_subti = "some service is down on mm-frontend02.phx2.fedoraproject.org"
    expected_icon = 'https://apps.fedoraproject.org/img/icons/nagios-logo.png'
    expected_secondary_icon = expected_icon
    expected_packages = set([])
    expected_usernames = set([])
    expected_objects = set([
        'mm-frontend02.phx2.fedoraproject.org/unknown',
    ])
    expected_link = "https://admin.fedoraproject.org/nagios/cgi-bin/" + \
        "status.cgi?navbarsearch=1&host=mm-frontend02.phx2.fedoraproject.org"
    msg = {
        "i": 1,
        "timestamp": 1450202769.0,
        "msg_id": "2015-071873c7-9635-48f4-bba3-1d97d87b0a76",
        "topic": "org.fedoraproject.prod.nagios.service.state.change",
        "msg": {
            "service_ack_author": "",
            "host": "mm-frontend02.phx2.fedoraproject.org",
            "type": "PROBLEM",
            "host_ack_author": "",
            "state": "CRITICAL"
        }
    }


class TestNagiosHostStateChange(Base):
    """ It might not be a good idea... but we hooked our `nagios instance
    <https://admin.fedoraproject.org/nagios>`_ because "open infrastructure".
    Note that, you shouldn't really rely on this stream for alerts about
    systems, because if you're expecting to get an alert about fedmsg being
    down, via fedmsg... well, it's just not going to work, friend.

    Here's an example message sent when a **host** changes state.
    """
    expected_title = "nagios.host.state.change"
    expected_subti = "proxy04.fedoraproject.org is down: \"ZOMG!\""
    expected_icon = 'https://apps.fedoraproject.org/img/icons/nagios-logo.png'
    expected_secondary_icon = expected_icon
    expected_packages = set([])
    expected_usernames = set([])
    expected_objects = set([
        'proxy04.fedoraproject.org/state',
    ])
    expected_link = "https://admin.fedoraproject.org/nagios/cgi-bin/" + \
        "status.cgi?navbarsearch=1&host=proxy04.fedoraproject.org"
    msg = {
        "i": 1,
        "timestamp": 1450127478.0,
        "msg_id": "2015-c29fd2e7-b4f3-4ac0-8eed-9aa2f6d0d675",
        "topic": "org.fedoraproject.prod.nagios.host.state.change",
        "msg": {
            "host": "proxy04.fedoraproject.org",
            "type": "PROBLEM",
            "state": "CRITICAL",
            "output": "ZOMG!",
            "host_ack_author": "",
            "service_ack_author": "",
        }
    }


class TestNagiosServiceStateChange(Base):
    """ It might not be a good idea... but we hooked our `nagios instance
    <https://admin.fedoraproject.org/nagios>`_ because "open infrastructure".
    Note that, you shouldn't really rely on this stream for alerts about
    systems, because if you're expecting to get an alert about fedmsg being
    down, via fedmsg... well, it's just not going to work, friend.

    Here's an example message sent when a **service** changes state.
    """
    expected_title = "nagios.service.state.change"
    expected_subti = "mm-publiclist-internal is back up on " + \
        "mm-frontend02.phx2.fedoraproject.org: " + \
        "\"HTTP OK: HTTP/1.1 200 OK - 438 bytes " + \
        "in 0.536 second response time\""
    expected_icon = 'https://apps.fedoraproject.org/img/icons/nagios-logo.png'
    expected_secondary_icon = expected_icon
    expected_packages = set([])
    expected_usernames = set([])
    expected_objects = set([
        'mm-frontend02.phx2.fedoraproject.org/mm-publiclist-internal',
    ])
    expected_link = "https://admin.fedoraproject.org/nagios/cgi-bin/" + \
        "status.cgi?navbarsearch=1&host=mm-frontend02.phx2.fedoraproject.org"
    msg = {
        "i": 1,
        "timestamp": 1450204405.0,
        "msg_id": "2015-6b240c0f-c946-40ea-b9d7-34237c3268fc",
        "topic": "org.fedoraproject.prod.nagios.service.state.change",
        "msg": {
            "service": "mm-publiclist-internal",
            "state": "OK",
            "host": "mm-frontend02.phx2.fedoraproject.org",
            "output": "HTTP OK: HTTP/1.1 200 OK - 438 bytes in 0.536 second response time",
            "type": "RECOVERY",
            "service_ack_author": "",
            "host_ack_author": ""
        }
    }


class TestNagiosServiceAck(Base):
    """ It might not be a good idea... but we hooked our `nagios instance
    <https://admin.fedoraproject.org/nagios>`_ because "open infrastructure".
    Note that, you shouldn't really rely on this stream for alerts about
    systems, because if you're expecting to get an alert about fedmsg being
    down, via fedmsg... well, it's just not going to work, friend.

    Here's an example message sent when an admin **acknowledges** a problem.
    """
    expected_title = "nagios.service.state.change"
    expected_subti = "kevin acknowledged that some service is having " + \
        "problems on packages03.phx2.fedoraproject.org"
    expected_icon = 'https://apps.fedoraproject.org/img/icons/nagios-logo.png'
    expected_secondary_icon = 'https://seccdn.libravatar.org/avatar/' + \
        '1a7d8c43c8b89789a33a3266b0e20be7759a502ff38b74ff724a4db6aa33ede8' + \
        '?s=64&d=retro'
    expected_packages = set([])
    expected_usernames = set(['kevin'])
    expected_objects = set([
        'packages03.phx2.fedoraproject.org/unknown',
    ])
    expected_link = "https://admin.fedoraproject.org/nagios/cgi-bin/" + \
        "status.cgi?navbarsearch=1&host=packages03.phx2.fedoraproject.org"
    msg = {
        "i": 1,
        "timestamp": 1450195509.0,
        "msg_id": "2015-e80ad530-8f35-473d-b90d-26b61de2f51d",
        "topic": "org.fedoraproject.prod.nagios.service.state.change",
        "msg": {
            "service_ack_author": "http://kevin.id.fedoraproject.org/",
            "host": "packages03.phx2.fedoraproject.org",
            "type": "ACKNOWLEDGEMENT",
            "host_ack_author": "",
            "state": "WARNING"
        }
    }


add_doc(locals())

if __name__ == '__main__':
    unittest.main()



# This file is part of fedmsg.
# Copyright (C) 2015 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
from fedmsg_meta_fedora_infrastructure import BaseProcessor
from fedmsg_meta_fedora_infrastructure.fasshim import avatar_url


class InfraGitProcessor(BaseProcessor):
    __name__ = "infragit"
    __description__ = "Fedora Infrastructure repos"
    __link__ = "https://infrastructure.fedoraproject.org/cgit/"
    __docs__ = "https://fedoraproject.org/wiki/Using_Fedora_GIT"
    __docs__ = "https://infrastructure.fedoraproject.org/infra/" + \
        "docs/infra-git-repo.rst"
    __obj__ = "Infrastructure Commits"
    __icon__ = "https://apps.fedoraproject.org/img/icons/git-logo.png"

    def _get_repo(self, msg):
        return msg['msg']['commit']['path'].split('/')[-1]

    def subtitle(self, msg, **config):
        repo = self._get_repo(msg)
        user = self.agent(msg, **config)

        summ = msg['msg']['commit']['summary']
        whole = msg['msg']['commit']['message']
        if summ.strip() != whole.strip():
            summ += " (..more)"

        branch = msg['msg']['commit']['branch']

        tmpl = self._('{user} pushed a commit to the fedora-infra {repo} repo '
                      '({branch}):  "{summary}"')
        return tmpl.format(user=user, repo=repo, branch=branch, summary=summ)

    def link(self, msg, **config):
        prefix = "https://infrastructure.fedoraproject.org/cgit"
        repo = self._get_repo(msg)

        # We don't have public links to some old repos..
        if repo in ['puppet']:
            return ''

        rev = msg['msg']['commit']['rev']
        branch = msg['msg']['commit']['branch']
        tmpl = "{prefix}/{repo}.git/commit/?h={branch}&id={rev}"
        return tmpl.format(prefix=prefix, repo=repo, branch=branch, rev=rev)

    def secondary_icon(self, msg, **config):
        return avatar_url(self.agent(msg, **config))

    def usernames(self, msg, **config):
        return set([self.agent(msg, **config)])

    def agent(self, msg, **config):
        return msg['msg']['commit']['username']

    def objects(self, msg, **config):
        repo = self._get_repo(msg)
        return set([
            repo + '/' + filename
            for filename in msg['msg']['commit']['stats']['files']
        ])



import fedmsg.tests.test_meta
import arrow

class TestBodhiConglomerateCommentSameUpdate(
        fedmsg.tests.test_meta.ConglomerateBase):
    originals = [
        {
            "i": 1,
            "msg": {
                "agent": "hreindl",
                "comment": {
                    "anonymous": False,
                    "author": "hreindl",
                    "group": None,
                    "karma": 1,
                    "text": "works for me",
                    "timestamp": 1408799774.0,
                    "update_request": None,
                    "update_status": "testing",
                    "update_submitter": "jstanek",
                    "update_title": "sqlite-3.8.6-2.fc20"
                }
            },
            "msg_id": "2014-10047900-1379-4ede-ba94-8319823541a8",
            "timestamp": 1408799776.0,
            "topic": "org.fedoraproject.prod.bodhi.update.comment"
        },
        {
            "i": 1,
            "msg": {
                "agent": "hreindl",
                "comment": {
                    "anonymous": False,
                    "author": "hreindl",
                    "group": None,
                    "karma": 1,
                    "text": "works for me",
                    "timestamp": 1408799744.0,
                    "update_request": None,
                    "update_status": "testing",
                    "update_submitter": "jkaluza",
                    "update_title": "file-5.19-4.fc20"
                }
            },
            "msg_id": "2014-c1ccc3bb-e9bc-4424-8b99-6fbfa24a128f",
            "timestamp": 1408799746.0,
            "topic": "org.fedoraproject.prod.bodhi.update.comment"
        },
        {
            "i": 1,
            "msg": {
                "agent": "volter",
                "comment": {
                    "anonymous": False,
                    "author": "volter",
                    "group": None,
                    "karma": 0,
                    "text": "Please add spatialite-tools-4.1.1-6.fc20",
                    "timestamp": 1408787443.0,
                    "update_request": None,
                    "update_status": "testing",
                    "update_submitter": "jstanek",
                    "update_title": "sqlite-3.8.6-2.fc20"
                }
            },
            "msg_id": "2014-f0205c2a-205e-4c79-aad3-542429b3fca8",
            "timestamp": 1408787444.0,
            "topic": "org.fedoraproject.prod.bodhi.update.comment"
        },

    ]
    expected = [{
        'subtitle': 'hreindl and volter commented on sqlite-3.8.6-2.fc20',
        'subjective': 'hreindl and volter commented on sqlite-3.8.6-2.fc20',
        'link': 'https://bodhi.fedoraproject.org/updates/sqlite-3.8.6-2.fc20',
        'icon': 'https://apps.fedoraproject.org/img/icons/bodhi.png',
        'secondary_icon': 'https://apps.fedoraproject.org/img/icons/bodhi.png',

        'start_time': 1408787444.0,
        'end_time': 1408799776.0,
        'timestamp': 1408793610.0,
        'human_time': arrow.get(1408793610.0).humanize(),

        'usernames': set(['volter', 'hreindl']),
        'packages': set(['sqlite']),
        'topics': set(['org.fedoraproject.prod.bodhi.update.comment']),
        'categories': set(['bodhi']),
        'msg_ids': [
            '2014-10047900-1379-4ede-ba94-8319823541a8',
            '2014-f0205c2a-205e-4c79-aad3-542429b3fca8',
        ],
    }, {
        'subtitle': u'hreindl commented on bodhi update '
        'file-5.19-4.fc20 (karma: 1)',
        'subjective': u'hreindl commented on bodhi update '
        'file-5.19-4.fc20 (karma: 1)',
        'link': 'https://bodhi.fedoraproject.org/updates/file-5.19-4.fc20',
        'icon': 'https://apps.fedoraproject.org/img/icons/bodhi.png',
        'secondary_icon': 'https://seccdn.libravatar.org/avatar/'
        'c82951c2d404ca0cfea20f51b040175ba4d1624f51e8ec366e75565b672e943d'
        '?s=64&d=retro',

        'start_time': 1408799746.0,
        'end_time': 1408799746.0,
        'timestamp': 1408799746.0,
        'human_time': arrow.get(1408799746.0).humanize(),

        'usernames': set(['hreindl']),
        'packages': set(['file']),
        'topics': set(['org.fedoraproject.prod.bodhi.update.comment']),
        'categories': set(['bodhi']),
        'msg_ids': ['2014-c1ccc3bb-e9bc-4424-8b99-6fbfa24a128f'],
    } ]


class TestBodhiConglomerateCommentSameUser(
        fedmsg.tests.test_meta.ConglomerateBase):
    originals = [
        {
            "i": 1,
            "msg": {
                "agent": "hreindl",
                "comment": {
                    "anonymous": False,
                    "author": "hreindl",
                    "group": None,
                    "karma": 1,
                    "text": "works for me",
                    "timestamp": 1408799774.0,
                    "update_request": None,
                    "update_status": "testing",
                    "update_submitter": "jstanek",
                    "update_title": "sqlite-3.8.6-2.fc20"
                }
            },
            "msg_id": "2014-10047900-1379-4ede-ba94-8319823541a8",
            "timestamp": 1408799776.0,
            "topic": "org.fedoraproject.prod.bodhi.update.comment"
        },
        {
            "i": 1,
            "msg": {
                "agent": "hreindl",
                "comment": {
                    "anonymous": False,
                    "author": "hreindl",
                    "group": None,
                    "karma": 1,
                    "text": "works for me",
                    "timestamp": 1408799744.0,
                    "update_request": None,
                    "update_status": "testing",
                    "update_submitter": "jkaluza",
                    "update_title": "file-5.19-4.fc20"
                }
            },
            "msg_id": "2014-c1ccc3bb-e9bc-4424-8b99-6fbfa24a128f",
            "timestamp": 1408799746.0,
            "topic": "org.fedoraproject.prod.bodhi.update.comment"
        },
    ]
    expected = [{
        'subtitle': 'hreindl commented on file-5.19-4.fc20 '
        'and sqlite-3.8.6-2.fc20',
        'subjective': 'hreindl commented on file-5.19-4.fc20 '
        'and sqlite-3.8.6-2.fc20',
        'link': 'https://bodhi.fedoraproject.org/users/hreindl',
        'icon': 'https://apps.fedoraproject.org/img/icons/bodhi.png',
        'secondary_icon': 'https://seccdn.libravatar.org/avatar/'
        'c82951c2d404ca0cfea20f51b040175ba4d1624f51e8ec366e75565b672e943d'
        '?s=64&d=retro',

        'start_time': 1408799746.0,
        'end_time': 1408799776.0,
        'timestamp': 1408799761.0,
        'human_time': arrow.get(1408799761.0).humanize(),

        'usernames': set(['hreindl']),
        'packages': set(['sqlite', 'file']),
        'topics': set(['org.fedoraproject.prod.bodhi.update.comment']),
        'categories': set(['bodhi']),
        'msg_ids': [
            '2014-10047900-1379-4ede-ba94-8319823541a8',
            '2014-c1ccc3bb-e9bc-4424-8b99-6fbfa24a128f',
        ],
    } ]
